//: /be/optiloading/gui/RecentMenu.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.prefs.Preferences;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import be.optiloading.settings.Settings;

/**
 * RecentMenu shows the most recent opened files.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class RecentMenu extends JMenu {
	private Preferences prefs;
	private JMenuItem erase;
	
	/**
	 * Constructor.
	 * @param value Text to display.
	 */
	public RecentMenu(String value){
		super(value);
		prefs = Preferences.userNodeForPackage(getClass());
		addSeparator();
		erase = new JMenuItem(Settings.getInstance().getResourceBundle()
				.getString("eraseAll"));
		
		erase.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				eraseAll();
			}
		});
		
		add(erase);
		loadList();
	}
	
	// Load the list from the preferences
	private void loadList(){
		String all = prefs.get("fileList", "");
		if(!all.equals("")){
			String[] toAdd = all.split(";");
			for(int i = (toAdd.length-1); -1 < i; i--){
				insertItem(toAdd[i]);
			}
		}
	}
	
	// Save the list to the preferences
	private void saveList(){
		String list = "";
		for(int i = 0; i < (getItemCount()-2);i++){
			list += ((FileItem) getMenuComponent(i)).getFilePath() +";";
		}
		prefs.put("fileList", list);
	}
	
	// Open a specific item
	private void openItem(FileItem item){
		File file = new File(item.getFilePath());
		remove(item);
		if(file.exists()){
			MainFrame.getInstance().openFile(file);
		} else {
			JOptionPane.showMessageDialog(MainFrame.getInstance(),
				Settings.getInstance().getResourceBundle().getString("fileNotFound"));
		}
	}
	
	/**
	 * Insert a new item in the recent opened menu.
	 * @param filePath The filePath of the recently opened file.
	 */
	public void insertItem(String filePath){
		removeSame(filePath);
		if(getItemCount() == 7){
			remove(4);
		}
		add(new FileItem(filePath), 0);
		saveList();
	}
	
	// Erase the whole list
	private void eraseAll(){
		removeAll();
		addSeparator();
		add(erase);
		prefs.put("fileList", "");
	}
	
	// Remove the files with the same filepath
	private void removeSame(String filePath){
		for(int i = 0; i < (getItemCount()-2);i++){
			if(((FileItem) getMenuComponent(i)).getFilePath().equals(filePath)){
				remove(i);
			}
		}
	}
	
	// MenuItems of the recent files menu.
	private class FileItem extends JMenuItem{
		private String filePath;
		private FileItem instance;
		private final int MAX_ITEM_LEN = 50;      
		private final String FILE_SEPARATOR_STR 
							= System.getProperty("file.separator");
		
		public FileItem(String filePath){
			this.filePath = filePath;
			instance = this;
			this.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent arg0) {
					openItem(instance);
				}
			});
			setText(getItemname(filePath));
		}
		
		public String getFilePath(){
			return filePath;
		}
		
		private String getItemname(String pathname) {
			final char FILE_SEPARATOR = FILE_SEPARATOR_STR.charAt(0);
			final int pathnameLen = pathname.length();
			// if the pathame is short enough: return whole pathname
			if (pathnameLen <= MAX_ITEM_LEN) {
				return pathname;
			}
			// if we have only one directory: return whole pathname
			// we will not cut to MAX_ITEM_LEN here
			if (pathname.indexOf(FILE_SEPARATOR_STR) == pathname
					.lastIndexOf(FILE_SEPARATOR_STR)) {
				return pathname;
			} else {
				// abbreviate pathanme: Windows OS like solution
				final int ABBREVIATED_PREFIX_LEN = 6; // e.g.: C:\..\
				final int MAX_PATHNAME_LEN = MAX_ITEM_LEN
						- ABBREVIATED_PREFIX_LEN;
				int firstFileSeparatorIndex = 0;
				for (int i = pathnameLen - 1; i >= (pathnameLen - 
						MAX_PATHNAME_LEN); i--) {
					if (pathname.charAt(i) == FILE_SEPARATOR) {
						firstFileSeparatorIndex = i;
					}
				}
				if (firstFileSeparatorIndex > 0) {
					return pathname.substring(0, 3)
							+ ".."
							+ pathname.substring(firstFileSeparatorIndex,
									pathnameLen);
				} else {
					return pathname.substring(0, 3)
							+ ".."
							+ FILE_SEPARATOR_STR
							+ ".."
							+ pathname
									.substring(pathnameLen - MAX_PATHNAME_LEN,
											pathnameLen);
				}
			}
		}
	}
}
