//: /be/optiloading/optimization/Lp.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.optimization;

import java.io.FileWriter;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.Arrays;
import java.util.ResourceBundle;

import javax.swing.JOptionPane;

import lpsolve.AbortListener;
import lpsolve.LogListener;
import lpsolve.LpSolve;

import org.gnu.glpk.GlpkHookIFC;
import org.gnu.glpk.GlpkSolver;

import be.optiloading.cargo.Cargo;
import be.optiloading.cargo.CargoList;
import be.optiloading.compatibility.CargoComp;
import be.optiloading.compatibility.SegregationComp;
import be.optiloading.compatibility.TankComp;
import be.optiloading.gui.Log;
import be.optiloading.gui.MainFrame;
import be.optiloading.gui.ProgressPanel;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;
import be.optiloading.ship.StabilityData;
import be.optiloading.tank.TankList;

/**
 * Lp to perform simplex algorithm on linear model.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class Lp extends Thread {
	private TankList cargoTanks;
	private CargoList cargoList;
	private TankComp tankCompList;
	private CargoComp cargoCompList;
	private SegregationComp segregationCompList;
	private ShipData data;
	private SectionList sectionList;
	private int i;
	private int j;
	private int z;
	private float displacement;
    private float lcb;
    private float mctc;
    private float lcf;
    private int tankfactorzero;
    private volatile boolean abortvalue;
    private ResourceBundle languageResource;
    
    //Settings
    private float desiredtrim;
    private float trimrelaxation;
    private float epsilon;
    public final static String GLPK = "GNU GLPK";
    public final static String LPSOLVE = "LP Solve";
    private String s;
    private boolean strength;
    private boolean usedtanks;
	private boolean trim;
	private boolean stability;
	private boolean heel;
	private boolean compatibility;
	private boolean debug;
	private float strengthfactor;
	private float usedtanksfactor;
	private long begintime;
	
	//Test Variables
	private static float percentfull = 0.25f;
	private static int numberofloads = 1;
	private static float tepsilon = 0.3f;
	private static boolean tstrength = true;
	private static boolean tusedtanks = true;
	private static boolean ttrim = false;
	private static boolean tstability = false;
	private static boolean theel = false;
	private static boolean tcompatibility = true;
	private static float ttrimrelaxation = 0.05f;
	private static float tusedtanksfactor = 1;
	private static float tstrengthfactor = 1;
	private static float tdesiredtrim = 0f;
	private static int numberofcomp = 1;
	private static boolean newloads = true; 
	private static float[] loads;
	private static String file;

	/**
	 * Test procedure
	 * @param args String arguments from commandline
	 */
    public static void main(String[] args){
    	file = args[0];
    	new Lp(Lp.LPSOLVE, true).run();
    	System.exit(0);
    }
    
    /**
     * Constructor
     * @param solver String indicting which solver to use.
     * @param debug Boolean value which is true when in debug mode.
     */
	public Lp(String solver, boolean debug){
		begintime = System.currentTimeMillis();
		s = solver;
		languageResource = Settings.getInstance().getResourceBundle();
		this.debug = debug;
		data = ShipData.getInstance();
		if(!debug){
			loadSettings();
		} else {
			loadTestProcedure();
		}
		cargoTanks = data.getCargoTankList();
		cargoList = data.getCargoList();
		tankCompList = data.getTankCompList();
		cargoCompList = data.getCargoCompList();
		segregationCompList = data.getSegregationCompList();
		cargoTanks.emptyTanks(true);
		i = cargoList.size();
		j = cargoTanks.size();
		float currdisplacement = data.getDisplacement();
	    displacement = cargoList.getTotalWeight() + 
	    data.getMiscTankList().getTotalWeight() + data.getLightship();
	    StabilityData stabData = data.getStabilityData();
	    stabData.setDisplacement(displacement);
	    stabData.setDontcheck(true);
	    lcb = stabData.getLcb();
	    mctc = stabData.getMctc();
	    lcf = stabData.getLcf();
	    stabData.setDontcheck(false);
	    stabData.setDisplacement(currdisplacement);
	    if(strength){
	    	sectionList = new SectionList(displacement, lcb, mctc, lcf, desiredtrim);
		    z = sectionList.size();
		    tankfactorzero = sectionList.getTankfactorzero();
	    } else {
	    	z = 0;
	    }
	}

	// Load test parameters and settings
	private void loadTestProcedure(){
		data.initializeShipData("/data/ship.xml");
		data.initializeStabilityData("/data/hydrostatic.xml", "/data/kn.xml");
		data.initializeStrengthData("/data/lightship.xml", "/data/buoyancy.xml");
		data.initializeCargoTankList("/data/cargotanks.xml");
		data.initializeBallastTankList("/data/ballasttanks.xml");
		data.initializeMiscTankList("/data/misctanks.xml");
		data.initializeCargoList();
		data.initializeCargoCompList();
		data.initializeTankCompList();
		
		float totalvol = 2353.376f*percentfull;
		if(numberofloads == 1){
			data.getCargoList().add(new Cargo("L0"
					, "Load 0", totalvol,1),null);
			data.getCargoCompList().addComp();
			data.getTankCompList().addComp();
		} 
		else {
			if(newloads){
				loads = generateLoadList(totalvol,numberofloads);
			}
			for(int k = 0; k < numberofloads; k++){
				data.getCargoList().add(new Cargo("L"+String.valueOf(k)
						, "Load "+String.valueOf(k), loads[k],1),null);
				data.getCargoCompList().addComp();
				data.getTankCompList().addComp();
			}
		}
		
		i = data.getCargoList().size();
		j = data.getCargoTankList().size();
		
		int countCargocomp = 0;
		int maxCargocomp = (int) (((i*i)-i)/2);
		int countTankcomp = 0;
		int maxTankcomp = i*j;
		for(int k = 0; k < numberofcomp; k++){
			if(Math.random() > 0.5f){
				if(countCargocomp >= maxCargocomp){
					boolean value = true;
					int horz = 0;
					int vert = 0;
					while(value){
						horz = new Float(Math.random()*i).intValue();
						vert = new Float(Math.random()*j).intValue();
						value = data.getTankCompList().get(horz, vert);
					}
					data.getTankCompList().set(true, horz, vert);
					countTankcomp++;
				} else {
					boolean value = true;
					int horz = 0;
					int vert = 0;
					while(value){
						horz = new Float(Math.random()*i).intValue();
						vert = new Float(Math.random()*i).intValue();
						if(horz != vert){
							value = data.getCargoCompList().get(horz, vert);
						}
					}
					data.getCargoCompList().set(true, horz, vert);
					countCargocomp++;
				}
			} else {
				if(countTankcomp >= maxTankcomp){
					boolean value = true;
					int horz = 0;
					int vert = 0;
					while(value){
						horz = new Float(Math.random()*i).intValue();
						vert = new Float(Math.random()*i).intValue();
						if(horz != vert){
							value = data.getCargoCompList().get(horz, vert);
						}
					}
					data.getCargoCompList().set(true, horz, vert);
					countCargocomp++;
				} else {
					boolean value = true;
					int horz = 0;
					int vert = 0;
					while(value){
						horz = new Float(Math.random()*i).intValue();
						vert = new Float(Math.random()*j).intValue();
						value = data.getTankCompList().get(horz, vert);
					}
					data.getTankCompList().set(true, horz, vert);
					countTankcomp++;
				}	
			}
		}
		
		epsilon = tepsilon;
		strength = tstrength;
		usedtanks = tusedtanks;
		usedtanks = tusedtanks;
		stability = tstability;
		heel = theel;
		trim = ttrim;
		compatibility = tcompatibility;
		trimrelaxation = ttrimrelaxation;
		usedtanksfactor = tusedtanksfactor;
		strengthfactor = tstrengthfactor;
		desiredtrim = tdesiredtrim;
	}
	
	// Generate random cargo objects
	private float[] generateLoadList(float totalsum, int parts){
		float[] splitters = new float[parts-1];
		for (int k = 0;k<numberofloads-1;k++) {
		    splitters[k] = (float)(Math.random()*totalsum);
		}
		Arrays.sort(splitters);
		float[] returnvalue = new float[parts];
		returnvalue[0] = splitters[0];
		for (int k = 0;k<parts-2;k++) {
			returnvalue[k+1] = (splitters[k+1] - splitters[k]);
		}
		returnvalue[parts-1] = (totalsum - splitters[parts-2]);
		return returnvalue;
	}
	
	// Load settings from Settings object
	private void loadSettings(){
		Settings settings = Settings.getInstance();
		epsilon = settings.getEpsilon();
		strength = settings.isStrength();
		usedtanks = settings.isUsedtanks();
		trim = settings.isTrim();
		stability = settings.isStability();
		heel = settings.isHeel();
		compatibility = settings.isCompatibility();
		trimrelaxation = settings.getTrimrelaxation();
		usedtanksfactor = settings.getUsedtanksfactor();
		strengthfactor = settings.getStrengthfactor();
		desiredtrim = settings.getDesiredTrim();
		settings = null;
	}
	
	/**
	 * Start Thread.
	 */
	public void run(){
		try{
		GlpkSolver solver = null;
		LpSolve lpsolve = null;
		if(s == GLPK){
			solver = new GlpkSolver();
			solver.setHook(new Hook());
		    solver.setProbName("OptiLoading");
		    solver.setObjDir(GlpkSolver.LPX_MIN);
		    solver.setClss(GlpkSolver.LPX_MIP);
		}
		
	    
	    /**
	     * Variables
	     */
	    int colsize = i*j*2;
	    if(stability){
	    	colsize += j;
	    }
	    if(strength){
	    	colsize += z;
	    }
	    if(s == GLPK){
	    	solver.addCols(colsize); 
	    } else {
	    	lpsolve = LpSolve.makeLp(0, colsize);
	    	LogListener logfunc = new LogListener() {
		        public void logfunc(LpSolve problem, Object handle, String buf) {
		        	if(debug){
		        		System.out.println(buf);
		        	} else {
		        		ProgressPanel.getInstance().addMsg(buf);
		        		Log.getInstance().add(Log.LP,buf);
		        	}
		        }
		    };
			lpsolve.putLogfunc(logfunc, null);
		    abortvalue = false;
		    AbortListener abortfunc = new AbortListener() {
		        public boolean abortfunc(LpSolve problem, Object handle) {
		        	return abortvalue;
		        }
		    };
			lpsolve.putAbortfunc(abortfunc, null);
	    }
	    
	    if(!debug){
	    	MainFrame.getInstance().lock();
	    	ProgressPanel.getInstance().clearProgress();
	 		ProgressPanel.getInstance().startCounter(this);
	    }
	   

	    int colcount = 1;
	    //Volume variables (i*j) >=0
	    for(int k1 = 1; k1 < (i+1);k1++){
	    	for(int k2 = 1; k2 < (j+1);k2++){
	    		if(s == GLPK){
	    			solver.setColName(colcount, "v"+String.valueOf(k1)
		    				+String.valueOf(k2)); 
				    solver.setColBnds(colcount, GlpkSolver.LPX_LO, 0, 0);
				    solver.setColKind(colcount, GlpkSolver.LPX_CV);
	    		} else {
	    			lpsolve.setColName(colcount, "v"+String.valueOf(k1)
		    				+String.valueOf(k2));
	    		}
	    		colcount++;
	    	}
	    }
	    //X variables (i*j) - binary
	    for(int k1 = 1; k1 < (i+1);k1++){
	    	for(int k2 = 1; k2 < (j+1);k2++){
	    		if(s == GLPK){
	    			solver.setColName(colcount, "x"+String.valueOf(k1)
		    				+String.valueOf(k2)); 
				    solver.setColBnds(colcount, GlpkSolver.LPX_DB, 0, 1);
				    solver.setColKind(colcount, GlpkSolver.LPX_IV);
				    if(usedtanks){
				    	solver.setObjCoef(colcount, usedtanksfactor/j);
				    }
	    		} else {
	    			lpsolve.setColName(colcount, "x"+String.valueOf(k1)
		    				+String.valueOf(k2));
	    			lpsolve.setBinary(colcount, true);
	    		}
	    		colcount++;
	    	}
	    }
	    //Y variables (j) - binary
	    if(stability){
	    for(int k = 1; k < (j+1);k++){
	    	if(s == GLPK){
	    		solver.setColName(colcount, "y"+String.valueOf(k)); 
		    	solver.setColBnds(colcount, GlpkSolver.LPX_DB, 0, 1);
				solver.setColKind(colcount, GlpkSolver.LPX_IV);
	    	} else {
    			lpsolve.setColName(colcount, "y"+String.valueOf(k));
    			lpsolve.setBinary(colcount, true);
    		}
    		colcount++;
	    }
	    }
	    //Z variables (z) >= 0
	    if(strength){
	    for(int k = 1; k < (z+1);k++){
	    	if(s == GLPK){
	    		solver.setColName(colcount, "z"+String.valueOf(k)); 
		    	solver.setColBnds(colcount, GlpkSolver.LPX_LO, 0, 0);
		    	solver.setColKind(colcount, GlpkSolver.LPX_CV);
				if(strength){
					solver.setObjCoef(colcount, 
							strengthfactor/(z*sectionList.get(k-1).getMaxforce()));
				}
	    	} else {
    			lpsolve.setColName(colcount, "z"+String.valueOf(k)); 
    		}
    		colcount++;
	    }
	    }
	    
	    /**
	     * Rows
	     */
	    int rowsize = i + j + (i*j);
	    int matrixsize = 1 + (4*i*j);
	    if(trim){
	    	if(trimrelaxation == 0f){
	    		rowsize += 1;
	    		matrixsize += i*j;
	    	} else {
	    		rowsize += 2;
	    		matrixsize += 2*i*j;
	    	}
	    }
	    if(heel){
	    	rowsize += 1;
	    	matrixsize += i*j;
	    }
	    if(stability){
	    	rowsize += 2*j;
	    	matrixsize += (2*i*j + 2*j);
	    }
	    if(compatibility){
	    	int loadloadcomb = (((i*i)-i)/2)*(j*j);
	    	rowsize += (loadloadcomb + i*j);
	    	matrixsize += (2*loadloadcomb + i*j);
	    }
	    if(strength){
	    	rowsize += 2*z;
	    	matrixsize += (2*z*i*j +2*z - tankfactorzero*i*2);
	    }
	    
	    int[] ia = null;
	    int[] ja = null;
	    double[] ar = null;
	    
	    int[] colno = null;
	    double[] rowno = null;
	    
	    if(s == GLPK){
	    	ia = new int[matrixsize];
		    ja = new int[matrixsize];
		    ar = new double[matrixsize];
	    	solver.addRows(rowsize);
	    } else {
	    	lpsolve.setAddRowmode(true);
	    	colno = new int[colsize+1];
		    rowno = new double[colsize+1];
	    }
	    colcount = 1;
	    int matrixcount = 1;
	    int count = 0;
	    //Sum of vol. = tot. vol. ( i ; i*j )
	    for(int k = 1; k < (i+1);k++){
	    	count = 0;
	    	for(int rowk = 1; rowk < (j+1); rowk++){
	    		if(s == GLPK){
	    			ia[matrixcount] = colcount;
			    	ja[matrixcount] = (k-1)*j+rowk;
			    	ar[matrixcount] = 1;
			    	matrixcount++;
	    		} else {
	    			colno[count] = (k-1)*j+rowk;
	    			rowno[count++] = 1;
	    		}
	    	}
	    	float load = cargoList.get(k-1).getVolume();
	    	if(s == GLPK){
	    		solver.setRowName(colcount, "load"+String.valueOf(k));
			    solver.setRowBnds(colcount, GlpkSolver.LPX_FX, load, load);
			    colcount++;
	    	} else {
	    		lpsolve.addConstraintex(count, rowno, colno, LpSolve.EQ, load);
	    	}
	    }
	    //Max. vol of tank ( i*j ; 2*i*j )
	    for(int k1= 1; k1 < (i+1);k1++){
	    	for(int k2= 1; k2 < (j+1);k2++){
	    		if(s == GLPK){
	    			ia[matrixcount] = colcount;
			    	ja[matrixcount] = ((k1-1)*j)+k2;
			    	ar[matrixcount] = 1;
			    	matrixcount++;
			    	ia[matrixcount] = colcount;
			    	ja[matrixcount] = ((k1-1)*j)+k2+(i*j);
			    	ar[matrixcount] = -cargoTanks.get(k2-1).getMaxvolume();
			    	matrixcount++;
		    		solver.setRowName(colcount, "maximum_vol"+
		    				String.valueOf(k1)+String.valueOf(k2));
				    solver.setRowBnds(colcount, GlpkSolver.LPX_UP, 0, 0);
				    colcount++;
	    		} else {
	    			count = 0;
	    			colno[count] = ((k1-1)*j)+k2;
	    			rowno[count++] = 1;
	    			colno[count] = ((k1-1)*j)+k2+(i*j);
	    			rowno[count++] = -cargoTanks.get(k2-1).getMaxvolume();
	    			lpsolve.addConstraintex(count, rowno, colno, LpSolve.LE, 0);
	    		}
	    	}
	    }
	    // One load per tank ( j ; i*j )
	    for(int k = 1; k < (j+1);k++){
	    	count = 0;
	    	for(int rowk = 1; rowk < (i+1); rowk++){
	    		if(s == GLPK){
	    			ia[matrixcount] = colcount;
		    		ja[matrixcount] = ((rowk-1)*j)+k+(i*j);
			    	ar[matrixcount] = 1;
			    	matrixcount++;
	    		} else {
	    			colno[count] = ((rowk-1)*j)+k+(i*j);
	    			rowno[count++] = 1;
	    		}
	    	}
	    	if(s == GLPK){
	    		solver.setRowName(colcount, "x_val"+String.valueOf(k));
			    solver.setRowBnds(colcount, GlpkSolver.LPX_UP, 0, 1);
			    colcount++;
	    	} else {
	    		lpsolve.addConstraintex(count, rowno, colno, LpSolve.LE, 1);
	    	}
	    }
	    // Trim ( 1 ; i*j)
	    if(trim){
	    //Relaxation O m
	    if(trimrelaxation == 0f){
	    	float dtrim = desiredtrim*100;
		    float lsmoment = data.getLightship() * data.getLightshiplcg();
		    float miscmoment = data.getMiscTankList().getLongMoment();
		    float C_trim = (displacement * lcb) - (dtrim*mctc) - (lsmoment+miscmoment);
		    if(s == GLPK){
		    	solver.setRowName(colcount, "trim");
			    solver.setRowBnds(colcount, GlpkSolver.LPX_FX, C_trim, C_trim);
		    } else{
		    	count = 0;
		    }
		    for(int k1= 1; k1 < (i+1);k1++){
		    	for(int k2= 1; k2 < (j+1);k2++){
		    		if(s == GLPK){
		    			ia[matrixcount] = colcount;
				    	ja[matrixcount] = ((k1-1)*j)+k2;
				    	ar[matrixcount] = cargoTanks.get(k2-1).getLCGMax()*cargoList
				    	.get(k1-1).getDensity();
				    	matrixcount++;
		    	    } else {
		    	    	colno[count] = ((k1-1)*j)+k2;
		    	    	rowno[count++] = cargoTanks.get(k2-1).getLCGMax()*cargoList
				    	.get(k1-1).getDensity();
		    	    }
		    	}
		    }
		    if(s == GLPK){
		    	colcount++;
		    } else {
		    	lpsolve.addConstraintex(count, rowno, colno, LpSolve.EQ, C_trim);
		    }
	    } 
	    //Relaxation != 0
	    else {
	    	float duptrim = (desiredtrim*100)+(trimrelaxation*50);
	    	float dlowtrim =  (desiredtrim*100)-(trimrelaxation*50);
		    float lsmoment = data.getLightship() * data.getLightshiplcg();
		    float miscmoment = data.getMiscTankList().getLongMoment();
		    float C_trim_up = (displacement * lcb) - (duptrim*mctc) 
		    					- (lsmoment+miscmoment);
		    float C_trim_low = (displacement * lcb) - (dlowtrim*mctc) 
			- (lsmoment+miscmoment);
		    float C_trim_max = 0f;
		    float C_trim_min = 0f;
		    if(C_trim_up > C_trim_low){
		    	C_trim_max = C_trim_up;
		    	C_trim_min = C_trim_low;
		    } else {
		    	C_trim_max = C_trim_low;
		    	C_trim_min = C_trim_up;
		    }
		    if(s == GLPK){
		    	solver.setRowName(colcount, "trim_max");
			    solver.setRowBnds(colcount, GlpkSolver.LPX_UP, 0, C_trim_max);
		    } else{
		    	count = 0;
		    }
		    for(int k1= 1; k1 < (i+1);k1++){
		    	for(int k2= 1; k2 < (j+1);k2++){
		    		if(s == GLPK){
		    			ia[matrixcount] = colcount;
				    	ja[matrixcount] = ((k1-1)*j)+k2;
				    	ar[matrixcount] = cargoTanks.get(k2-1).getLCGMax()*cargoList
				    	.get(k1-1).getDensity();
				    	matrixcount++;
		    	    } else {
		    	    	colno[count] = ((k1-1)*j)+k2;
		    	    	rowno[count++] = cargoTanks.get(k2-1).getLCGMax()*cargoList
				    	.get(k1-1).getDensity();
		    	    }
		    	}
		    }
		    if(s == GLPK){
		    	colcount++;
		    } else {
		    	lpsolve.addConstraintex(count, rowno, colno, LpSolve.LE, C_trim_max);
		    }
		    if(s == GLPK){
		    	solver.setRowName(colcount, "trim_min");
			    solver.setRowBnds(colcount, GlpkSolver.LPX_LO, C_trim_min, 0);
		    } else{
		    	count = 0;
		    }
		    for(int k1= 1; k1 < (i+1);k1++){
		    	for(int k2= 1; k2 < (j+1);k2++){
		    		if(s == GLPK){
		    			ia[matrixcount] = colcount;
				    	ja[matrixcount] = ((k1-1)*j)+k2;
				    	ar[matrixcount] = cargoTanks.get(k2-1).getLCGMax()*cargoList
				    	.get(k1-1).getDensity();
				    	matrixcount++;
		    	    } else {
		    	    	colno[count] = ((k1-1)*j)+k2;
		    	    	rowno[count++] = cargoTanks.get(k2-1).getLCGMax()*cargoList
				    	.get(k1-1).getDensity();
		    	    }
		    	}
		    }
		    if(s == GLPK){
		    	colcount++;
		    } else {
		    	lpsolve.addConstraintex(count, rowno, colno, LpSolve.GE, C_trim_min);
		    }
	    }
		}
	    //Heel (1; i*j)
	    if(heel){
	    float lstransmoment = data.getLightship() * data.getLightshiptcg();
	    float misctransmoment = data.getMiscTankList().getTransMoment();
	    float C_heel = -(lstransmoment + misctransmoment);
	    if(s == GLPK){
	    	solver.setRowName(colcount, "heel");
		    solver.setRowBnds(colcount, GlpkSolver.LPX_FX, C_heel, C_heel);
	    } else {
	    	count = 0;
	    }
	    for(int k1= 1; k1 < (i+1);k1++){
	    	for(int k2= 1; k2 < (j+1);k2++){
	    		if(s == GLPK){
	    			ia[matrixcount] = colcount;
			    	ja[matrixcount] = ((k1-1)*j)+k2;
			    	ar[matrixcount] = cargoTanks.get(k2-1).getTCGMax()*cargoList
			    	.get(k1-1).getDensity();
			    	matrixcount++;
	    	    } else {
	    	    	colno[count] = ((k1-1)*j)+k2;
	    	    	rowno[count++] = cargoTanks.get(k2-1).getTCGMax()*cargoList
			    	.get(k1-1).getDensity();
	    	    }
	    	}
	    }
	    if(s == GLPK){
	    	colcount++;
	    } else {
	    	lpsolve.addConstraintex(count, rowno, colno, LpSolve.EQ, C_heel);
	    }
	    }
	    // Load compatible with compartment ( i*j ; i*j )
	    if(compatibility){
	    for(int k1= 1; k1 < (i+1);k1++){
	    	for(int k2= 1; k2 < (j+1);k2++){
	    		count = 0;
	    		if(s == GLPK){
	    			ia[matrixcount] = colcount;
			    	ja[matrixcount] = ((k1-1)*j)+k2+j*i;
			    	ar[matrixcount] = 1;
			    	matrixcount++;
	    	    } else {
	    	    	colno[count] = ((k1-1)*j)+k2+j*i;
	    	    	rowno[count++] = 1;
	    	    }
	    		boolean comp = tankCompList.get(k1-1, k2-1);
	    		int sub = 0;
	    		if(comp){
	    			sub = 1;
	    		}
	    		if(s == GLPK){
	    			solver.setRowName(colcount, "loadcomp"+
		    				String.valueOf(k1)+String.valueOf(k2));
				    solver.setRowBnds(colcount, GlpkSolver.LPX_UP, 0, 1-sub);
				    colcount++;
	    	    } else {
	    	    	lpsolve.addConstraintex(count, rowno, colno, LpSolve.LE, 1-sub);
	    	    }
	    	}
	    }
	    // Load segregated from load (loadloadcomb ; 2*loadloadcomb)
	    for(int k1= 1; k1 < (i+1);k1++){
	    for(int k2= k1+1; k2 < (i+1);k2++){
	    	for(int k3= 1; k3 < (j+1);k3++){
	    	for(int k4= 1; k4 < (j+1);k4++){
	    		count = 0;
	    		if(s == GLPK){
	    			ia[matrixcount] = colcount;
				    ja[matrixcount] = ((k1-1)*j)+k3+(j*i);
				    ar[matrixcount] = 1;
			    	matrixcount++;
			    	ia[matrixcount] = colcount;
				    ja[matrixcount] = ((k2-1)*j)+k4+(j*i);
				    ar[matrixcount] = 1;
			    	matrixcount++;
	    	    } else {
	    	    	colno[count] = ((k1-1)*j)+k3+(j*i);
	    	    	rowno[count++] = 1;
	    	    	colno[count] = ((k2-1)*j)+k4+(j*i);
	    	    	rowno[count++] = 1;
	    	    }
	    		
		    	boolean loadload = cargoCompList.get(k1-1,k2-1);
		    	boolean tanktank = segregationCompList.get(k3-1,k4-1);
		    	int sub1 = 0;
		    	int sub2 = 0;
		    	if(loadload){
		    		sub1 = 1;
		    	}
		    	if(tanktank){
		    		sub2 = 1;
		    	}
		    	if(s == GLPK){
		    		solver.setRowName(colcount, "loadload"+
			    			String.valueOf(k1)+String.valueOf(k2));
			    	solver.setRowBnds(colcount, GlpkSolver.LPX_UP, 0, 3-sub1-sub2);
			    	colcount++;
			    } else {
			    	lpsolve.addConstraintex(count, rowno, colno, 
			    			LpSolve.LE, 3-sub1-sub2);
			    }
	    	}
	    	}
	    }
		}
	    }
		//20-80 rule (2*j;2*i*j+j)
	    if(stability){
	    for(int k = 1; k < (j+1); k++){
	    	count = 0;
	    	for(int rowk = 1; rowk < (i+1); rowk++){
	    		if(s == GLPK){
	    			ia[matrixcount] = colcount;
		    		ja[matrixcount] = ((rowk-1)*j)+k;
			    	ar[matrixcount] = -1/cargoTanks.get(k-1).getMax();
			    	matrixcount++;
	    	    } else {
	    	    	colno[count] = ((rowk-1)*j)+k;
	    	    	rowno[count++] = -1/cargoTanks.get(k-1).getMax();
	    	    }
	    	}
	    	if(s == GLPK){
	    		ia[matrixcount] = colcount;
	    		ja[matrixcount] = (2*i*j)+k;
		    	ar[matrixcount] = epsilon + 0.5f;
		    	matrixcount++;
		    	solver.setRowName(colcount, "firstabs"+String.valueOf(k));
			    solver.setRowBnds(colcount, GlpkSolver.LPX_LO, epsilon-0.5f, 0);
			    colcount++;
		    } else {
		    	colno[count] = (2*i*j)+k;
    	    	rowno[count++] = epsilon + 0.5f;
    	    	lpsolve.addConstraintex(count, rowno, colno, 
    	    			LpSolve.GE, epsilon-0.5f);
		    }
	    }
	    for(int k = 1; k < (j+1); k++){
	    	count = 0;
	    	for(int rowk = 1; rowk < (i+1); rowk++){
	    		if(s == GLPK){
	    			ia[matrixcount] = colcount;
		    		ja[matrixcount] = ((rowk-1)*j)+k;
			    	ar[matrixcount] = 1/cargoTanks.get(k-1).getMax();
			    	matrixcount++;
	    	    } else {
	    	    	colno[count] = ((rowk-1)*j)+k;
	    	    	rowno[count++] = 1/cargoTanks.get(k-1).getMax();
	    	    }
	    	}
	    	if(s == GLPK){
	    		ia[matrixcount] = colcount;
	    		ja[matrixcount] = (2*i*j)+k;
		    	ar[matrixcount] = -(0.5f + epsilon);
		    	matrixcount++;
		    	solver.setRowName(colcount, "secondabs"+String.valueOf(k));
			    solver.setRowBnds(colcount, GlpkSolver.LPX_LO, 0, 0);
			    colcount++;
		    } else {
		    	colno[count] = (2*i*j)+k;
    	    	rowno[count++] = -(0.5f + epsilon);
    	    	lpsolve.addConstraintex(count, rowno, colno, LpSolve.GE, 0);
		    }
	    }
	    }
	    //Stress constraints ( 2*z ; 2*z*i*j)
	    if(strength){
	    for(int k1 = 1; k1 <(z+1);k1++){
	    	count = 0;
	    	for(int k2= 1; k2 < (i+1);k2++){
	    		for(int k3= 1; k3 < (j+1);k3++){
	    			if(sectionList.get(k1-1).getTankfactors(k3-1) != 0f){
	    				if(s == GLPK){
	    					ia[matrixcount] = colcount;
			    			ja[matrixcount] = ((k2-1)*j)+k3;
			    			ar[matrixcount] = cargoList.get(k2-1).getDensity()*
			    			sectionList.get(k1-1).getTankfactors(k3-1) ;
			    			matrixcount++;
	    			    } else {
	    			    	colno[count] = ((k2-1)*j)+k3;
	    	    	    	rowno[count++] = cargoList.get(k2-1).getDensity()*
			    			sectionList.get(k1-1).getTankfactors(k3-1) ;
	    			    }
	    			}
	    		}
	    	}
	    	if(s == GLPK){
	    		ia[matrixcount] = colcount;
	    		if(stability){
	    			ja[matrixcount] = (2*i*j)+k1+j;
	    		} else {
	    			ja[matrixcount] = (2*i*j)+k1;
	    		}
		    	ar[matrixcount] = -1;
		    	matrixcount++;
		    	solver.setRowName(colcount, "zpos"+String.valueOf(k1));
		    	solver.setRowBnds(colcount, GlpkSolver.LPX_UP, 0, 
		    			-sectionList.get(k1-1).getBuoymisc());
		    	colcount++;
		    } else {
		    	if(stability){
		    		colno[count] = (2*i*j)+k1+j;
	    		} else {
	    			colno[count] = (2*i*j)+k1;
	    		}
    	    	rowno[count++] = -1;
    	    	lpsolve.addConstraintex(count, rowno, colno, LpSolve.LE, 
    	    			-sectionList.get(k1-1).getBuoymisc());
		    }
	    }
	    for(int k1 = 1; k1 <(z+1);k1++){
	    	count = 0;
	    	for(int k2= 1; k2 < (i+1);k2++){
	    		for(int k3= 1; k3 < (j+1);k3++){
	    			if(sectionList.get(k1-1).getTankfactors(k3-1) != 0f){
	    				if(s == GLPK){
	    					ia[matrixcount] = colcount;
			    			ja[matrixcount] = ((k2-1)*j)+k3;
			    			ar[matrixcount] = - cargoList.get(k2-1).getDensity()*
			    				sectionList.get(k1-1).getTankfactors(k3-1) ;
			    			matrixcount++;
	    			    } else {
	    			    	colno[count] = ((k2-1)*j)+k3;
	    	    	    	rowno[count++] = - cargoList.get(k2-1).getDensity()*
			    				sectionList.get(k1-1).getTankfactors(k3-1) ;
	    			    }
	    			}
	    		}
	    	}
	    	if(s == GLPK){
	    		ia[matrixcount] = colcount;
	    		if(stability){
	    			ja[matrixcount] = (2*i*j)+k1+j;
	    		} else {
	    			ja[matrixcount] = (2*i*j)+k1;
	    		}
		    	ar[matrixcount] = -1;
		    	matrixcount++;
		    	solver.setRowName(colcount, "neg"+String.valueOf(k1));
		    	solver.setRowBnds(colcount, GlpkSolver.LPX_UP, 0, 
		    			sectionList.get(k1-1).getBuoymisc());
		    	colcount++;
		    } else {
		    	if(stability){
		    		colno[count] = (2*i*j)+k1+j;
	    		} else {
	    			colno[count] = (2*i*j)+k1;
	    		}
    	    	rowno[count++] = -1;
    	    	lpsolve.addConstraintex(count, rowno, colno, LpSolve.LE, 
    	    			sectionList.get(k1-1).getBuoymisc());
		    }
	    }
	    }
	    
	    if(!debug){
    		ProgressPanel.getInstance().addMsg(
    				languageResource.getString("numberOfVariables") + ": "
    				+ String.valueOf(colsize));
    		ProgressPanel.getInstance().addMsg(
    				languageResource.getString("numberOfConstraints") + ": "
    				+ String.valueOf(rowsize));
    	}
	    
	    // Print Solution
	    if(s == GLPK){
	    	solver.loadMatrix(matrixcount-1, ia, ja, ar);
	    	if(!debug){
	    		ProgressPanel.getInstance().addMsg(
	    				languageResource.getString("glpkNoOutput"));
	    	}
		    solver.simplex();
			solver.integer();
			boolean found = false;
		    for(int k = 1;k<(i*j)+1;k++){
		    	int load = (int) (k-1) / j;
		    	int tank = k-(load*j)-1;
		    	if(tank == -1){
		    		tank = j-1;
		    	}
		    	if(solver.mipColVal(k) >= 0.006f){
		    		found = true;
		    		cargoTanks.get(tank).setVolume((float) solver.mipColVal(k), 
		    				cargoList.get(load).getDensity());
		    		cargoTanks.get(tank).setCargoid(
		    				cargoList.get(load).getCargoid());
		    	}
		    }
		    if(debug){
				double[] variables = new double[colsize];
		    	for(int k = 1;k<(colsize+1);k++){
			    	if(solver.mipColVal(k) >= 0.006f){
			    		System.out.print(solver.getColName(k) + " ");
				    	System.out.println(solver.mipColVal(k));
			    	}
			    	variables[k-1] = solver.mipColVal(k);
			    }
				System.out.println("Z MIP = "+solver.mipObjVal());
				writeReport(variables);
			}
		    solver.deleteProb();
		    unlockInterface();
		    if(!found){
		    	JOptionPane.showMessageDialog(MainFrame.getInstance(),
		    			languageResource.getString("notFound"));
		    }
	    } else {
	    	lpsolve.setAddRowmode(false);
	    	
	    	//Set Objective function
	    	count = 0;
	    	if(usedtanks){
	    		for(int k = 1; k < (i*j)+1;k++){
		    		colno[count] = k + (i*j);
		    		rowno[count++] = usedtanksfactor/j;
	    		}
	    	}
	    	if(strength){
	    		if(stability){
	    		for(int k = 1; k < (z+1);k++){
		    		colno[count] = k + ((2*i*j)+j);
		    		rowno[count++] = strengthfactor/(z*sectionList.get(k-1).getMaxforce());
		    	}
	    		}else{
	    		for(int k = 1; k < (z+1);k++){
			    	colno[count] = k + (2*i*j);
			    	rowno[count++] = strengthfactor/(z*sectionList.get(k-1).getMaxforce());
			    }	
	    		}
	    	}
	    	lpsolve.setObjFnex(count, rowno, colno);
	    	lpsolve.setMinim();
	    	
	    	lpsolve.setVerbose(LpSolve.NORMAL);

	    	switch(lpsolve.solve()){
	    		case LpSolve.NOMEMORY:
	    			unlockInterface();
	    			if(debug == false){
	    			JOptionPane.showMessageDialog(MainFrame.getInstance(),
	    					languageResource.getString("outofmemMessage"));
	    			} else {
	    				writeReport("No memory");
	    			}
	    			break;
	    		case LpSolve.OPTIMAL:
	    			lpsolve.getVariables(rowno);
	    			for(int k = 1;k<(i*j)+1;k++){
	    		    	int load = (int) (k-1) / j;
	    		    	int tank = k-(load*j)-1;
	    		    	if(tank == -1){
	    		    		tank = j-1;
	    		    	}
	    		    	if(rowno[k-1] >= 0.006f){
	    		    		cargoTanks.get(tank).setVolume((float) rowno[k-1], 
	    		    				cargoList.get(load).getDensity());
	    		    		cargoTanks.get(tank).setCargoid(
	    		    				cargoList.get(load).getCargoid());
	    		    	}
	    		    }
	    			if(debug){
	    				System.out.println("Z: " + lpsolve.getObjective());
	    		    	for(int k = 0;k<colsize;k++){
	    			    	if(rowno[k] != 0f){
	    			    		System.out.print(lpsolve.getColName(k+1) + " ");
	    				    	System.out.println(rowno[k]);
	    			    	}
	    			    }
	    		    	writeReport(rowno);
	    			}
	    			unlockInterface();
	    			break;
	    		case LpSolve.SUBOPTIMAL:
	    			lpsolve.getVariables(rowno);
	    			if(debug){
	    				System.out.println("Z: " + lpsolve.getObjective());
	    				for(int k = 0;k<colsize;k++){
	    					if(rowno[k] != 0f){
	    						System.out.print(lpsolve.getColName(k+1) + " ");
	    						System.out.println(rowno[k]);
	    					}
	    				}
	    			}
	    			for(int k = 1;k<(i*j)+1;k++){
	    				int load = (int) (k-1) / j;
	    				int tank = k-(load*j)-1;
	    				if(tank == -1){
	    					tank = j-1;
	    				}
	    				if(rowno[k-1] >= 0.006f){
	    					cargoTanks.get(tank).setVolume((float) rowno[k-1], 
	    							cargoList.get(load).getDensity());
	    					cargoTanks.get(tank).setCargoid(
	    							cargoList.get(load).getCargoid());
	    				}
	    			}
	    			unlockInterface();
	    			if(debug){
	    				writeReport(rowno);
	    			} else {
	    				JOptionPane.showMessageDialog(
	    	    				MainFrame.getInstance(),
	    	    				languageResource.getString("suboptimalMessage"));
	    			}
	    			break;
	    		case LpSolve.INFEASIBLE: 
	    			unlockInterface();
	    			if(debug == false){
	    			JOptionPane.showMessageDialog(MainFrame.getInstance(),
	    					languageResource.getString("infeasableMessage"));
	    			} else {
	    				writeReport("Infeasable");
	    			}
	    			break;
	    		case LpSolve.UNBOUNDED: 
	    			unlockInterface();
	    			if(debug == false){
	    			JOptionPane.showMessageDialog(MainFrame.getInstance(),
	    					languageResource.getString("unboundedMessage"));
	    			} else {
	    				writeReport("Unbounded");
	    			}
	    			break;
	    		case LpSolve.DEGENERATE: 
	    			unlockInterface();
	    			if(debug == false){
	    			JOptionPane.showMessageDialog(MainFrame.getInstance(),
	    					languageResource.getString("degenerateMessage"));
	    			} else {
	    				writeReport("Degenerate");
	    			}
	    			break;
	    		case LpSolve.NUMFAILURE: 
	    			unlockInterface();
	    			if(debug == false){
	    			JOptionPane.showMessageDialog(MainFrame.getInstance(),
	    					languageResource.getString("numfailureMessage"));
	    			} else {
	    				writeReport("Numerical failure");
	    			}
	    			break;
	    		case LpSolve.FEASFOUND:
	    			lpsolve.getVariables(rowno);
	    			if(debug){
	    				System.out.println("Z: " + lpsolve.getObjective());
	    		    	for(int k = 0;k<colsize;k++){
	    			    	if(rowno[k] != 0f){
	    			    		System.out.print(lpsolve.getColName(k+1) + " ");
	    				    	System.out.println(rowno[k]);
	    			    	}
	    			    }
	    			}
	    			for(int k = 1;k<(i*j)+1;k++){
	    		    	int load = (int) (k-1) / j;
	    		    	int tank = k-(load*j)-1;
	    		    	if(tank == -1){
	    		    		tank = j-1;
	    		    	}
	    		    	if(rowno[k-1] >= 0.006f){
	    		    		cargoTanks.get(tank).setVolume((float) rowno[k-1], 
	    		    				cargoList.get(load).getDensity());
	    		    		cargoTanks.get(tank).setCargoid(
	    		    				cargoList.get(load).getCargoid());
	    		    	}
	    		    }
	    			if(debug){
	    				writeReport(rowno);
	    			}
	    			unlockInterface();
	    			break;
	    		case LpSolve.NOFEASFOUND: 
	    			unlockInterface();
	    			if(debug == false){
	    				JOptionPane.showMessageDialog(MainFrame.getInstance(),
	    						languageResource.getString("infeasableMessage"));
	    			} else {
	    				writeReport("No Feasable found");
	    			}
	    			break;
	    		default: 
	    			unlockInterface();
	    			if(debug == false){
	    				JOptionPane.showMessageDialog(MainFrame.getInstance(),
	    						languageResource.getString("lperrorMessage")); 
	    			} else {
	    				writeReport("General error");
	    			}
	    	}
	    	lpsolve.deleteLp();
	    }
	    }catch(Exception e){
			e.printStackTrace();
			JOptionPane.showMessageDialog(MainFrame.getInstance(),
					languageResource.getString("exceptionMessage"));
			Log.getInstance().add(Log.LP, e);
			if(debug == false){
				MainFrame.getInstance().unlock();
		    	ProgressPanel.getInstance().stopCounter();
			}
		} catch (UnsatisfiedLinkError e){
			if(s == GLPK){
				e.printStackTrace();
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						languageResource.getString("glpknotfound"));
			} else {
				e.printStackTrace();
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						languageResource.getString("lpsolvenotfound"));
			}			
		}
	}
	
	/**
	 * Abort algorithm execution.
	 */
	public void abort(){
		abortvalue = true;
	}
	
	// Unlock interface
	private void unlockInterface(){
		if(debug == false){
	    	MainFrame.getInstance().setTabbedPane(2);
	    	MainFrame.getInstance().updateInterface();
	    	MainFrame.getInstance().unlock();
	    }
	}
	
	// Write output to file
	private void writeReport(double[] variables){
		NumberFormat formatter = NumberFormat.getInstance();
        formatter.setMaximumFractionDigits(0);
        formatter.setMinimumFractionDigits(0);
        String numberofloads = formatter.format(i);
        
        
        float totalz = 0f;
        if(stability){
        	for(int k = 0; k < z; k++){
        		totalz += variables[(2*i*j)+j+k];
        	}
        } else {
        	for(int k = 0; k < z; k++){
        		totalz += variables[(2*i*j)+k];
        	}
        }
        String totalstrength = formatter.format(totalz);
        
        String imo = "nee";
        if((data.getGm()>=0.15f) && (data.getArea30() >= 0.055f) && 
        		(data.getArea40()>=0.09f) && (data.getArea30to40()>= 0.03f) &&
        		(data.getMaxGZAngle() > 30f) && (data.getGZ30() >= 0.2f)){
        	imo = "ja";
        }
        
        String objective = "";
        if(usedtanks){
        	objective += ("\\#T"+formatter.format(usedtanksfactor))+ " ";
        }
        if(strength){
        	objective += ("BL"+formatter.format(strengthfactor))+ " ";
        }
        String constraints = "";
        if(compatibility){
        	constraints +=  ("CO"+formatter.format(numberofcomp))+ " ";
        }
        formatter.setMaximumFractionDigits(4);
        formatter.setMinimumFractionDigits(4);
        if(trim){
        	constraints +=  ("TR"+formatter.format(trimrelaxation))+ " ";
        }
        if(heel){
        	constraints +=  "SL ";
        }
        if(stability){
        	constraints +=  ("ST"+formatter.format(epsilon))+ " ";
        }
        
        String totalpercentfull = formatter.format(percentfull);
        String trimdev = formatter.format(Math.abs(data.getTrim() - desiredtrim));
        String heeldev = formatter.format(Math.abs(data.getHeel()));
        
        String distribution = "";
        for(int k = 1;k<(i*j)+1;k++){
			int load = (int) (k-1) / j;
			int tank = k-(load*j)-1;
			if(tank == -1){
				tank = j-1;
			}
			if(variables[k-1] != 0f){
				float value = (float) (variables[k-1]/cargoTanks.get(tank).
						getMax());
				distribution += "\\textcolor{load"+String.valueOf(load)+"}{"
						+formatter.format(value)+"} ";
			}
		}
        
        Long time = System.currentTimeMillis() - begintime;
        String cputime = formatter.format(time.floatValue()/60000);
        
        String lpsolver = "LPS";
        if(s == Lp.GLPK){
        	lpsolver = "GLPK";
        }
        
        
		try {
			FileWriter fwriter = new FileWriter(file,true);
			
			fwriter.write(numberofloads+" & "+ totalpercentfull 
					+ " & " + objective + " & "+ constraints + " & " +
					lpsolver + " & " + trimdev + " & "+ heeldev + " & " +
					totalstrength + " & " + imo + " & {\\scriptsize" +
					distribution + "} & " + cputime + "\\\\ \n");
			fwriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	// Write output to file
	private void writeReport(String error){
		NumberFormat formatter = NumberFormat.getInstance();
        formatter.setMaximumFractionDigits(0);
        formatter.setMinimumFractionDigits(0);
        String numberofloads = formatter.format(i);
        
        String objective = "";
        if(usedtanks){
        	objective += ("\\#T"+formatter.format(usedtanksfactor))+ " ";
        }
        if(strength){
        	objective += ("BL"+formatter.format(strengthfactor))+ " ";
        }
        String constraints = "";
        if(compatibility){
        	constraints +=  ("CO"+formatter.format(numberofcomp))+ " ";
        }
        formatter.setMaximumFractionDigits(2);
        formatter.setMinimumFractionDigits(2);
        if(trim){
        	constraints +=  ("TR"+formatter.format(trimrelaxation))+ " ";
        }
        if(heel){
        	constraints +=  "SL ";
        }
        if(stability){
        	constraints +=  ("ST"+formatter.format(epsilon))+ " ";
        }
        
        Long time = System.currentTimeMillis() - begintime;
        String cputime = formatter.format(time.floatValue()/60000);
        
        String lpsolver = "LPS";
        if(s == Lp.GLPK){
        	lpsolver = "GLPK";
        }
        
		try {
			FileWriter fwriter = new FileWriter("/Users/Tim/report.txt",true);
			
			fwriter.write(numberofloads+" & " + " & " + objective + 
					" & "+ constraints + " & " + lpsolver + " & " + 
					error + " & " + cputime + "\\\\ \n");
			fwriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	// Class for GLPK log.
	private class Hook implements GlpkHookIFC{

		public void fault(String s) {
			JOptionPane.showMessageDialog(null,s);	
		}

		public void print(String s) {
			JOptionPane.showMessageDialog(null,s);
		}
		
	}	
}
