//: /be/optiloading/optimization/SectionList.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.optimization;

import java.util.ArrayList;
import java.util.Collections;
import java.util.TreeSet;

import be.optiloading.cargo.CargoList;
import be.optiloading.ship.ShipData;
import be.optiloading.tank.Tank;
import be.optiloading.tank.TankList;

/**
 * SectionList to hold an ArrayList of Sections.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class SectionList extends ArrayList<Section>{
	private TankList cargoTanks;
	private CargoList cargoList;
	private int i;
	private int j;
	private int tankfactorzero;
	
	/**
	 * Constructor.
	 * @param displacement Displacement.
	 * @param lcb LCB at desired condition.
	 * @param mctc MCTC at desired condition.
	 * @param lcf LCF at desired condition.
	 * @param desiredtrim Desired trim
	 */
	public SectionList(float displacement, float lcb, float mctc, 
			float lcf, float desiredtrim){
		cargoTanks = ShipData.getInstance().getCargoTankList();
		cargoList = ShipData.getInstance().getCargoList();
		i = cargoList.size();
		j = cargoTanks.size();
		TreeSet<Bound> boundList = new TreeSet<Bound>();
		tankfactorzero = 0;
		for(Tank tank: cargoTanks){
			boundList.add(new Bound(tank.getAftbound()));
			boundList.add(new Bound(tank.getFwdbound()));
		}

		ArrayList<Float> boundList2 = new ArrayList<Float>();
		for(Bound b: boundList){
			boundList2.add(new Float(b.get()));
		}

		for(int k = 0; k < (boundList2.size()-1);k++){
			float aft = boundList2.get(k);
			float fwd = boundList2.get(k+1);
			float[] tankfactors = new float[j];
			for(int k2 = 0; k2 < j;k2++){
				float a = cargoTanks.get(k2).getAftbound();
				float b = cargoTanks.get(k2).getFwdbound();
				float seclength = fwd - aft;
				float aftbound = 0f;
				float fwdbound = 0f;
				if((a>=fwd)||(b <= aft)){
					tankfactors[k2] = 0;
					tankfactorzero++;
				} else {
					if(a>=aft){
						aftbound = a;
					} else {
						aftbound = aft;
					}
					if(b<=fwd){
						fwdbound = b;
					} else {
						fwdbound = fwd;
					}
					float length = fwdbound - aftbound;
					tankfactors[k2] = (length/seclength);
				}
			}
			
			float deslcg = lcb - (mctc*desiredtrim*100)/displacement;
			float buoymisc = ShipData.getInstance().getBuoymisc(aft, fwd, lcb, 
					deslcg, lcf, displacement);
			
			ArrayList<TankVolume> tankVolumes = new ArrayList<TankVolume>();
			float totalVolume = 0f;
			for(int k2 = 0; k2 < j; k2++){
				if(tankfactors[k2] != 0){
					tankVolumes.add(new TankVolume(k2,
							tankfactors[k2]*cargoTanks.get(k2).getMaxvolume()));
					totalVolume += tankfactors[k2]*cargoTanks.get(k2).getMaxvolume();
				}
			}
			Collections.sort(tankVolumes);
			
			ArrayList<CargoWeight> cargoWeights = new ArrayList<CargoWeight>();
			for(int k2 = 0; k2 < i; k2++){
				cargoWeights.add(new CargoWeight(k2,cargoList.get(k2).getWeight()));
			}
			Collections.sort(cargoWeights);
			float maxweight = 0;
			if(cargoList.get(cargoWeights.get(cargoWeights.size()-1).
					getCargo()).getVolume() >=totalVolume){
				maxweight = totalVolume*cargoList.get(cargoWeights.get
						(cargoWeights.size()-1).getCargo()).getDensity();
			} else {
				for(int k2 = tankVolumes.size()-1;k2>-1;k2--){
					float vol = tankVolumes.get(k2).getVolume();
					if(cargoList.get(cargoWeights.get(cargoWeights.size()-1).
							getCargo()).getVolume() >=vol){
						float weight = vol * cargoList.get(cargoWeights.get
								(cargoWeights.size()-1).getCargo()).getDensity();
						maxweight +=weight;
						cargoWeights.get(cargoWeights.size()-1).subWeight(weight);
						Collections.sort(cargoWeights);
					} else {
						maxweight += cargoWeights.get(cargoWeights.size()-1).getWeight();
						cargoWeights.remove(cargoWeights.size()-1);
					}
					if(cargoWeights.size() == 0){
						break;
					}
				}
			}
			
			float maxforce = Math.abs(buoymisc);
			
			if(Math.abs(maxweight + buoymisc) > Math.abs(buoymisc)){
				maxforce = maxweight + buoymisc;
			}
			
			this.add(new Section(fwd, aft, tankfactors, buoymisc, maxforce));
		}
	}
	
	// Class to get all the bounds for the different sections.
	private class Bound implements Comparable<Bound>{
		Float value;
		
		public Bound(float value){
			this.value = value;
		}
		
		public int hashCode(){
			return value.hashCode();
		}
		
		public boolean equals(Object o){
			Bound b = (Bound) o;
			return value.equals(b.get());
		}
		
		public float get(){
			return value;
		}

		public int compareTo(Bound o) {
			int returnvalue = -1;
			if(value.floatValue() == o.get()){
				returnvalue = 0;
			}
			if(value.floatValue()>o.get()){
				returnvalue = 1;
			}
			return returnvalue;
		}
	}
	
	// Class for the different tank volumes
	private class TankVolume implements Comparable<TankVolume>{
		private int tank;
		private float volume;
		
		public TankVolume(int tank, float volume){
			this.tank = tank;
			this.volume = volume; 
		}

		public int getTank() {
			return tank;
		}

		public float getVolume() {
			return volume;
		}

		public int compareTo(TankVolume o) {
			int returnvalue = -1;
			if(volume == o.getVolume()){
				returnvalue = 0;
			}
			if(volume>o.getVolume()){
				returnvalue = 1;
			}
			return returnvalue;
		}
		public String toString(){
			return String.valueOf(tank) + " " + String.valueOf(volume);
		}
	}
	
	// Class for the different cargo weights
	private class CargoWeight implements Comparable<CargoWeight>{
		private int cargo;
		private float weight;
		
		public CargoWeight(int cargo, float weight){
			this.cargo = cargo;
			this.weight = weight; 
		}

		public int getCargo() {
			return cargo;
		}

		public float getWeight() {
			return weight;
		}
		
		public void subWeight(float f){
			weight = weight -f;
		}

		public int compareTo(CargoWeight o) {
			int returnvalue = -1;
			if(weight == o.getWeight()){
				returnvalue = 0;
			}
			if(weight >o.getWeight()){
				returnvalue = 1;
			}
			return returnvalue;
		}
		public String toString(){
			return String.valueOf(cargo) + " " + String.valueOf(weight);
		}
	}

	/**
	 * Get how many tank factors are zero.
	 * @return Integer value containing how many tank factors are zero.
	 */
	public int getTankfactorzero() {
		return tankfactorzero;
	}
}
