//: be/optiloading/gui/Settings.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.settings;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.NumberFormat;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import be.optiloading.gui.Log;
import be.optiloading.gui.MainFrame;
import be.optiloading.ship.ShipData;
import be.optiloading.ship.ShipParticular;

/**
 * Settings class creates the settings panel.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class Settings{
	private static Settings uniqueInstance;
	private GeneticPanel geneticpanel;
	private GeneralPanel generalpanel;
	private LpPanel lppanel;
	private SaPanel sapanel;
	
	private float crossrate;
	private float mutrate;
	private int popsize;
	private float eliterate;
	private int restarts;
	private int optimum;
	private int creation;
	private int selection;
	private int crossover;
	private int mutation;
	private boolean strength;
	private boolean trim;
	private boolean stability;
	private boolean usedtanks;
	private boolean heel;
	private boolean compatibility;
	private float epsilon;
	private float trimrelaxation;
	private int strengthfactor;
	private int usedtanksfactor;
	private float density;
	private float ballastdensity;
	private float desiredtrim;
	private float desiredheel;
	private int language;
	private int defaultlang;
	private float saheelfactor;
	private float sacompatibilityfactor;
	private float satrimfactor;
	private float sausedtanksfactor;
	private float sastrengthfactor;
	private boolean saheel;
	private boolean sacompatibility;
	private boolean satrim;
	private boolean sausedtanks;
	private boolean sastrength;
	private float coolingrate;
	private int trailspertemp;
	private int goodswapspertemp;
	private int maxiterations;
	private float starttempfactor;
	private int numberofrestarts;
	private String solver;
	
	private Preferences prefs;
	
	private NumberFormat formatter;
	private float maxdeadweight;
	
	static ResourceBundle languageResource;
	
	private SettingsPanel panel;
	private SettingsDialog dialog;
	
	/**
	 * Returns the uniqueInstance of the Settings object.
	 * 
	 * @return Settings object.
	 */
	public static Settings getInstance() {
		if (uniqueInstance == null) {
			uniqueInstance = new Settings();
		}
		return uniqueInstance;
	}

	// Private constructor (singleton)
	private Settings() {
		prefs = Preferences.userNodeForPackage(getClass());
		
		Locale defaultlanguage = Locale.getDefault();
		defaultlang = 0;
		if(defaultlanguage.getISO3Language().equals("nld") || 
				defaultlanguage.getISO3Language().equals("dut")){
			defaultlang = 1;
		}
		language = prefs.getInt("language", defaultlang);
		
		languageResource = ResourceBundle.getBundle("languages.language", getSystemLocale());
		formatter = NumberFormat.getInstance();
        formatter.setMaximumFractionDigits(2);
        formatter.setMinimumFractionDigits(2);
		panel = new SettingsPanel();
		loadSettings();
	}
	
	private class SettingsPanel extends JPanel{
		public SettingsPanel(){
			setLayout(new BorderLayout());
			// Make TabbedPanes
			geneticpanel = new GeneticPanel();
			generalpanel = new GeneralPanel();
			lppanel = new LpPanel();
			sapanel = new SaPanel();
			
			JTabbedPane tabbed = new JTabbedPane();
			tabbed.addTab(languageResource.getString("generalTitle"),generalpanel);
			//tabbed.add(languageResource.getString("geneticTitle"), geneticpanel);
			tabbed.addTab(languageResource.getString("lpTitle"), lppanel);
			tabbed.addTab(languageResource.getString("saTitle"), sapanel);
			
			//Button panel
			JPanel buttonPanel = new JPanel(new BorderLayout());
			
			JPanel okandcancel = new JPanel();
			JButton ok = new JButton("OK");
			ok.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent arg0){
					if(checkDensity() == true){
						if(checkLanguage() == false){
							saveSettings();
							JOptionPane.showMessageDialog(dialog, 
									ResourceBundle.getBundle("languages.language", 
											getSystemLocale()).getString("messageLanguage"));
						} else {
							saveSettings();
						}
						dialog.setVisible(false);
						MainFrame.getInstance().updateInterface();
					} else {
						JOptionPane.showMessageDialog(dialog, 
								String.format(languageResource.getString("messageDeadweight"), 
										formatter.format(maxdeadweight)));
						generalpanel.waterdensfield.setValue(density);
					}
				}
			});
			
			JButton cancel = new JButton(languageResource.getString("cancelLabel"));
			cancel.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent arg0) {
					dialog.setVisible(false);
				}
			});
			
			okandcancel.add(cancel);
			okandcancel.add(ok);
			
			JPanel defaultspanel = new JPanel();
			
			JButton defaults = new JButton(languageResource.getString("restoreLabel"));
			defaults.setToolTipText(languageResource.getString("restoreTooltip"));
			defaults.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent arg0) {
					int i = JOptionPane.showConfirmDialog(dialog,
							languageResource.getString("messageRestore"),
							languageResource.getString("messageRestoreTitle"),
							JOptionPane.YES_NO_OPTION);
					if(i==0){
						generalpanel.waterdensfield.setValue(new Float(1.025f));
						
						if(checkDensity() == true){
							int previouslang = language;
							try {
								prefs.clear();
							} catch (BackingStoreException e) {
								e.printStackTrace();
								Log.getInstance().add(Log.JAVA,e);
								JOptionPane.showMessageDialog(dialog, 
										languageResource.getString("exceptionMessage"),
										languageResource.getString("exceptionTitle"),
										JOptionPane.WARNING_MESSAGE);
							}
							loadSettings();
							if(previouslang != defaultlang){
								JOptionPane.showMessageDialog(dialog, 
										ResourceBundle.getBundle("languages.language", 
												getLocale()).getString("messageLanguage"));
							}
							MainFrame.getInstance().updateInterface();
						} else {
							JOptionPane.showMessageDialog(dialog, 
									String.format(languageResource.
											getString("messageRestoreDeadweight"), 
											formatter.format(maxdeadweight)));
							generalpanel.waterdensfield.setValue(density);
						}
					}
				}
			});
			defaultspanel.add(defaults);
			
			buttonPanel.add(BorderLayout.WEST,defaultspanel);
			buttonPanel.add(BorderLayout.EAST,okandcancel);
			
			// Add to panel
			add(BorderLayout.CENTER, tabbed);
			add(BorderLayout.SOUTH,buttonPanel);
			
		}
	}
	
	private class SettingsDialog extends JDialog{
		public SettingsDialog(){
			super(MainFrame.getInstance(), true);
			setTitle(languageResource.getString("settingsTitle"));
			getContentPane().add(panel);
			setResizable(false);
			pack();
			setLocationRelativeTo(null);
			
			this.addWindowListener(new WindowAdapter(){
				public void windowDeactivated(WindowEvent e){
					loadSettings();
				}
			});

		}
	}
	
	private void loadSettings(){
		crossrate = prefs.getFloat("crossrate", 0.7f);
		geneticpanel.crossratefield.setValue(crossrate);
		geneticpanel.crossrate.setValue(new Float(crossrate*100).intValue());
		
		mutrate = prefs.getFloat("mutrate", 0.1f);
		geneticpanel.mutratefield.setValue(mutrate);
		geneticpanel.mutrate.setValue(new Float(mutrate*100).intValue());
		
		popsize = prefs.getInt("popsize", 500);
		geneticpanel.popsizefield.setValue(popsize);
		geneticpanel.popsize.setValue(popsize);
		
		eliterate = prefs.getFloat("eliterate", 0.02f);
		geneticpanel.eliteratefield.setValue(eliterate);
		geneticpanel.eliterate.setValue(new Float(eliterate*100).intValue());
		
		restarts = prefs.getInt("restarts", 5);
		geneticpanel.restarts.setValue(restarts);
		
		optimum = prefs.getInt("optimum", 25);
		geneticpanel.optimum.setValue(optimum);
		
		creation = prefs.getInt("creation", 1);
		geneticpanel.creationbox.setSelectedIndex(creation);
		
		selection = prefs.getInt("selection", 0);
		geneticpanel.selectionbox.setSelectedIndex(selection);
		
		crossover = prefs.getInt("crossover", 3);
		geneticpanel.crossoverbox.setSelectedIndex(crossover);
		
		mutation = prefs.getInt("mutation", 0);
		geneticpanel.mutationbox.setSelectedIndex(mutation);
		
		strength = prefs.getBoolean("strength", true);
		lppanel.strength.setSelected(strength);
		
		trim = prefs.getBoolean("trim", true);
		lppanel.trim.setSelected(trim);
		
		stability = prefs.getBoolean("stability", true);
		lppanel.stability.setSelected(stability);
		
		heel = prefs.getBoolean("heel", true);
		lppanel.heel.setSelected(heel);
		
		usedtanks = prefs.getBoolean("usedtanks", true);
		lppanel.usedtanks.setSelected(usedtanks);
		
		compatibility = prefs.getBoolean("compatibility", true);
		lppanel.compatibility.setSelected(compatibility);
		
		density = prefs.getFloat("density", 1.025f);
		generalpanel.waterdensfield.setValue(density);
		
		epsilon = prefs.getFloat("epsilon", 0.3f);
		lppanel.stabfield.setValue(epsilon);
		
		strengthfactor = prefs.getInt("strengthfactor", 1);
		lppanel.strengthfield.setValue(strengthfactor);
		
		usedtanksfactor = prefs.getInt("usedtanksfactor", 1);
		lppanel.usedtanksfield.setValue(usedtanksfactor);
		
		trimrelaxation = prefs.getFloat("trimrelaxation", 0.00f);
		lppanel.trimfield.setValue(trimrelaxation);
		
		ballastdensity = prefs.getFloat("ballastdensity", 1.025f);
		generalpanel.ballastwaterdensfield.setValue(ballastdensity);
		
		desiredtrim = prefs.getFloat("desiredtrim", 0.00f);
		generalpanel.inputtrim.setValue(desiredtrim);
		
		desiredheel = prefs.getFloat("desiredheel", 0.00f);
		generalpanel.inputheel.setValue(desiredheel);
		
		language = prefs.getInt("language", defaultlang);
		generalpanel.languagebox.setSelectedIndex(language);
		
		saheel = prefs.getBoolean("saheel", true);
		sapanel.heel.setSelected(saheel);
		
		sacompatibility = prefs.getBoolean("sacompatibility", true);
		sapanel.compatibility.setSelected(sacompatibility);
		
		satrim = prefs.getBoolean("satrim", true);
		sapanel.trim.setSelected(satrim);
		
		sausedtanks = prefs.getBoolean("sausedtanks", true);
		sapanel.usedtanks.setSelected(sausedtanks);
		
		sastrength = prefs.getBoolean("sastrength", true);
		sapanel.strength.setSelected(sastrength);
		
		saheelfactor = prefs.getFloat("saheelfactor", 1);
		sapanel.heelfield.setValue(saheelfactor);
		
		sacompatibilityfactor = prefs.getFloat("sacompatibilityfactor", 1);
		sapanel.compatibilityfield.setValue(sacompatibilityfactor);
		
		satrimfactor = prefs.getFloat("satrimfactor", 1);
		sapanel.trimfield.setValue(satrimfactor);
		
		sausedtanksfactor = prefs.getFloat("sausedtanksfactor", 1);
		sapanel.usedtanksfield.setValue(sausedtanksfactor);
		
		sastrengthfactor = prefs.getFloat("sastrengthfactor", 1);
		sapanel.strengthfield.setValue(sastrengthfactor);
		
		coolingrate = prefs.getFloat("coolingrate", 0.95f);
		sapanel.coolingratefield.setValue(coolingrate);
		
		trailspertemp = prefs.getInt("trailspertemp", 300);
		sapanel.trailspertempfield.setValue(trailspertemp);
		
		goodswapspertemp = prefs.getInt("goodswapspertemp", 100);
		sapanel.goodswapspertempfield.setValue(goodswapspertemp);
		
		maxiterations = prefs.getInt("maxiterations", 1000);
		sapanel.maxiterationsfield.setValue(maxiterations);
		
		starttempfactor = prefs.getFloat("starttempfactor", 100);
		sapanel.starttempfactorfield.setValue(starttempfactor);
		
		numberofrestarts = prefs.getInt("numberofrestarts", 10);
		sapanel.numberofrestartsfield.setValue(numberofrestarts);
		
		solver = prefs.get("solver", "LP Solve");
		lppanel.solverbox.setSelectedItem(new String(solver));
	}
	
	private void saveSettings(){
		crossrate = ((Number) geneticpanel.crossratefield.getValue()).floatValue();
		prefs.putFloat("crossrate", crossrate);
		
		mutrate = ((Number) geneticpanel.mutratefield.getValue()).floatValue();
		prefs.putFloat("mutrate", mutrate);
		
		popsize = (Integer) geneticpanel.popsizefield.getValue();
		prefs.putInt("popsize", popsize);
		
		eliterate = ((Number) geneticpanel.eliteratefield.getValue()).floatValue();
		prefs.putFloat("eliterate", eliterate);
		
		restarts = (Integer) geneticpanel.restarts.getValue();
		prefs.putInt("restarts", restarts);
		
		optimum = (Integer) geneticpanel.optimum.getValue();
		prefs.putInt("optimum", optimum);
		
		selection = geneticpanel.selectionbox.getSelectedIndex();
		prefs.putInt("selection", selection);
		
		crossover = geneticpanel.crossoverbox.getSelectedIndex();
		prefs.putInt("crossover", crossover);
		
		mutation = geneticpanel.mutationbox.getSelectedIndex();
		prefs.putInt("mutation", mutation);
		
		strength = lppanel.strength.isSelected();
		prefs.putBoolean("strength", strength);
		
		trim = lppanel.trim.isSelected();
		prefs.putBoolean("trim", trim);
		
		heel = lppanel.heel.isSelected();
		prefs.putBoolean("heel", heel);
		
		usedtanks = lppanel.usedtanks.isSelected();
		prefs.putBoolean("usedtanks", usedtanks);
		
		compatibility = lppanel.compatibility.isSelected();
		prefs.putBoolean("compatibility", compatibility);
		
		stability = lppanel.stability.isSelected();
		prefs.putBoolean("stability", stability);
		
		density = ((Number) generalpanel.waterdensfield.getValue()).floatValue();
		prefs.putFloat("density", density);
		
		epsilon = ((Number) lppanel.stabfield.getValue()).floatValue();
		prefs.putFloat("epsilon", epsilon);
		
		strengthfactor = ((Number) lppanel.strengthfield.getValue()).intValue();
		prefs.putInt("strengthfactor", strengthfactor);
		
		usedtanksfactor = ((Number) lppanel.usedtanksfield.getValue()).intValue();
		prefs.putInt("usedtanksfactor", usedtanksfactor);
		
		trimrelaxation = ((Number) lppanel.trimfield.getValue()).floatValue();
		prefs.putFloat("trimrelaxation", trimrelaxation);
		
		ballastdensity = ((Number) generalpanel.ballastwaterdensfield.getValue()).floatValue();
		prefs.putFloat("ballastdensity", ballastdensity);
		
		desiredtrim = ((Number) generalpanel.inputtrim.getValue()).floatValue();
		prefs.putFloat("desiredtrim", desiredtrim);
		
		desiredheel = ((Number) generalpanel.inputheel.getValue()).floatValue();
		prefs.putFloat("desiredheel", desiredheel);
		
		language = generalpanel.languagebox.getSelectedIndex();
		prefs.putInt("language", language);
		
		satrim = sapanel.trim.isSelected();
		prefs.putBoolean("satrim", satrim);
		
		sausedtanks = sapanel.usedtanks.isSelected();
		prefs.putBoolean("sausedtanks", sausedtanks);
		
		sastrength = sapanel.strength.isSelected();
		prefs.putBoolean("sastrength", sastrength);
		
		saheel = sapanel.heel.isSelected();
		prefs.putBoolean("saheel", saheel);
		
		sacompatibility = sapanel.compatibility.isSelected();
		prefs.putBoolean("sacompatibility", sacompatibility);
		
		saheelfactor = ((Number) sapanel.heelfield.getValue()).floatValue();
		prefs.putFloat("saheelfactor", saheelfactor);
		
		sacompatibilityfactor = ((Number) sapanel.compatibilityfield.getValue()).floatValue();
		prefs.putFloat("sacompatibilityfactor", sacompatibilityfactor);
		
		satrimfactor = ((Number) sapanel.trimfield.getValue()).floatValue();
		prefs.putFloat("satrimfactor", satrimfactor);
		
		sausedtanksfactor = ((Number) sapanel.usedtanksfield.getValue()).floatValue();
		prefs.putFloat("sausedtanksfactor", sausedtanksfactor);
		
		sastrengthfactor = ((Number) sapanel.strengthfield.getValue()).floatValue();
		prefs.putFloat("sastrengthfactor", sastrengthfactor);
		
		coolingrate = ((Number) sapanel.coolingratefield.getValue()).floatValue();
		prefs.putFloat("coolingrate", coolingrate);
		
		trailspertemp = ((Number) sapanel.trailspertempfield.getValue()).intValue();
		prefs.putInt("trailspertemp", trailspertemp);
		
		numberofrestarts = ((Number) sapanel.numberofrestartsfield.getValue()).intValue();
		prefs.putInt("numberofrestarts", numberofrestarts);
		
		goodswapspertemp = ((Number) sapanel.goodswapspertempfield.getValue()).intValue();
		prefs.putInt("goodswapspertemp", goodswapspertemp);
		
		maxiterations = ((Number) sapanel.maxiterationsfield.getValue()).intValue();
		prefs.putInt("maxiterations",maxiterations);
		
		starttempfactor = ((Number) sapanel.starttempfactorfield.getValue()).floatValue();
		prefs.putFloat("starttempfactor",starttempfactor);
		
		solver = (String) lppanel.solverbox.getSelectedItem();
		prefs.put("solver", solver);
	}
	
	private boolean checkDensity(){
		float dens = ((Number) generalpanel.waterdensfield.getValue()).floatValue();
		ShipData data = ShipData.getInstance();
		maxdeadweight = ((data.getLightship()+data.getMaxdeadweightinseawater())
				*(dens/1.025f))-data.getLightship();
		
		boolean returnvalue = true;
		
		if(data.getShipParticular(ShipParticular.DEADWEIGHT)>maxdeadweight){
			returnvalue = false;
		}
		
		return returnvalue;
	}

	private boolean checkLanguage(){
		boolean returnvalue = true;
		
		Locale defaultlanguage = Locale.getDefault();
		int lang = 0;
		if(defaultlanguage.getISO3Language().equals("nld") || 
				defaultlanguage.getISO3Language().equals("dut")){
			lang = 1;
		}

		if(generalpanel.languagebox.getSelectedIndex() != prefs.getInt("language", lang)){
			returnvalue = false;
		}
		
		return returnvalue;
	}
	
	/**
	 * Method to show the settings dialog.
	 */
	public void showDialog(){
		if(dialog == null){
			dialog = new SettingsDialog();
		}
		dialog.setVisible(true);
	}

	/**
	 * Method to get the crossover rate.
	 * 
	 * @return Floating point value containing the crossover rate.
	 */
	public float getCrossrate() {
		return crossrate;
	}

	/**
	 * Method to get the mutation rate.
	 * 
	 * @return Floating point value containing the mutation rate.
	 */
	public float getMutationrate() {
		return mutrate;
	}

	/**
	 * Method to get the population size.
	 * 
	 * @return Integer value containing the population size.
	 */
	public int getPopulationsize() {
		return popsize;
	}
	
	/**
	 * Method to get the elitism rate.
	 * 
	 * @return Integer value containing the elitism rate.
	 */
	public float getEliterate() {
		return eliterate;
	}

	/**
	 * Method to get the number of restarts.
	 * 
	 * @return Integer value containing the number of restarts.
	 */
	public int getRestarts() {
		return restarts;
	}

	/**
	 * Method to get the number of simular consecutive solutions before restart.
	 * 
	 * @return Integer value containing the number of simular consecutive
	 *         solutions before restart.
	 */
	public int getOptimum() {
		return optimum;
	}

	/**
	 * Method to get the creation method.
	 * 
	 * @return Integer value containing the index value of the creation method.
	 */
	public int getCreation() {
		return creation;
	}

	/**
	 * Method to get the selection method.
	 * 
	 * @return Integer value containing the index value of the selection method.
	 */
	public int getSelection() {
		return selection;
	}

	/**
	 * Method to get the crossover method.
	 * 
	 * @return Integer value containing the index value of the crossover method.
	 */
	public int getCrossover() {
		return crossover;
	}

	/**
	 * Method to get the mutation method.
	 * 
	 * @return Integer value containing the index value of the mutation method.
	 */
	public int getMutation() {
		return mutation;
	}

	/**
	 * Method to check if the strength criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isStrength() {
		return strength;
	}

	/**
	 * Method to check if the trim criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isTrim() {
		return trim;
	}
	
	/**
	 * Method to check if the heel criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isHeel() {
		return heel;
	}
	
	/**
	 * Method to check if the used tanks criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isUsedtanks() {
		return usedtanks;
	}
	
	/**
	 * Method to check if the stability criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isCompatibility() {
		return compatibility;
	}
	
	/**
	 * Method to check if the stability criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isStability() {
		return stability;
	}
	
	/**
	 * Method to check if the heel criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isSaHeel() {
		return saheel;
	}
	
	/**
	 * Method to check if the used tanks criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isSaUsedtanks() {
		return sausedtanks;
	}
	
	/**
	 * Method to check if the stability criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isSaCompatibility() {
		return sacompatibility;
	}
	
	/**
	 * Method to check if the strength criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isSaStrength() {
		return sastrength;
	}

	/**
	 * Method to check if the trim criteria is selected
	 * 
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isSaTrim() {
		return satrim;
	}
	
	/**
	 * Method to get the density of the water.
	 * @return Floating point value containing the density of the water.
	 */
	public float getDensity(){
		return  density;
	}
	
	/**
	 * Method to get the epsilon value.
	 * @return Floating point value containing the epsilon value.
	 */
	public float getEpsilon(){
		return  epsilon;
	}
	
	/**
	 * Method to get the trim relaxation value.
	 * @return Floating point value containing the trim relaxation value.
	 */
	public float getTrimrelaxation(){
		return  trimrelaxation;
	}
	
	/**
	 * Method to get the used tanks factor
	 * @return Integer value containing the used tanks factor.
	 */
	public int getUsedtanksfactor(){
		return  usedtanksfactor;
	}
	
	/**
	 * Method to get the number of restarts
	 * @return Integer value containing the number of restarts.
	 */
	public int getNumberOfRestarts(){
		return  numberofrestarts;
	}
	
	/**
	 * Method to get the strength factor
	 * @return Integer value containing the used strength factor.
	 */
	public int getStrengthfactor(){
		return  strengthfactor;
	}
	
	/**
	 * Method to get the used tanks factor
	 * @return Float value containing the used tanks factor.
	 */
	public float getSaUsedtanksfactor(){
		if(sausedtanks){
			return  sausedtanksfactor;
		} else {
			return 0f;
		}
		
	}
	
	/**
	 * Method to get the strength factor
	 * @return Float value containing the used strength factor.
	 */
	public float getSaStrengthfactor(){
		if(sastrength){
			return  sastrengthfactor;
		} else {
			return 0f;
		}
	}
	
	/**
	 * Method to get the trim factor
	 * @return Float value containing the used trim factor.
	 */
	public float getSaTrimfactor(){
		if(satrim){
			return  satrimfactor;
		} else {
			return 0f;
		}
	}
	
	/**
	 * Method to get the heel factor
	 * @return Float value containing the used heel factor.
	 */
	public float getSaHeelfactor(){
		if(saheel){
			return  saheelfactor;
		} else {
			return 0f;
		}
	}
	
	/**
	 * Method to get the compatibility factor
	 * @return Float value containing the used compatibility factor.
	 */
	public float getSaCompatibilityfactor(){
		if(sacompatibility){
			return  sacompatibilityfactor;
		} else {
			return 0f;
		}
	}
	
	/**
	 * Method to get the desired trim.
	 * @return Floating point value containing the desired trim.
	 */
	public float getDesiredTrim(){
		return  desiredtrim;
	}
	
	/**
	 * Method to get the cooling rate.
	 * @return Floating point value containing the cooling rate.
	 */
	public float getCoolingrate(){
		return coolingrate;
	}
	
	/**
	 * Method to get the trails per temp.
	 * @return Integer value containing the trails per temp.
	 */
	public int getTrailsPerTemp(){
		return trailspertemp;
	}
	
	/**
	 * Method to get the good swaps per temp.
	 * @return Integer value containing the good swaps per temp.
	 */
	public int getGoodSwapsPerTemp(){
		return goodswapspertemp;
	}
	
	/**
	 * Method to get the max iterations
	 * @return Integer value containing the max iterations.
	 */
	public int getMaxIterations(){
		return maxiterations;
	}
	
	/**
	 * Method to get the start temp factor.
	 * @return Floating point value containing the start temp factor.
	 */
	public float getStartTempFactor(){
		return starttempfactor;
	}
	
	/**
	 * Method to get the density of the ballast water.
	 * @return Floating point value containing the density of the water.
	 */
	public float getBallastDensity(){
		return  ballastdensity;
	}
	
	/**
	 * Method to get the desired trim.
	 * @return Floating point value containing the desired trim.
	 */
	public float getDesiredHeel(){
		return  desiredheel;
	}
	
	/**
	 * Method to get the locale
	 * @return Locale with the set language.
	 */
	public Locale getSystemLocale(){
		Locale returnvalue = null;
		switch(language){
		case 0: returnvalue = Locale.ENGLISH; break;
		case 1: returnvalue = new Locale("nl"); break;
		}
		return returnvalue;
	}
	
	/**
	 * Get the language.
	 * @return 0 for English. 1 for Dutch.
	 */
	public int getLanguage() {
		return language;
	}

	/**
	 * Get the used LP Solver
	 * @return String with LP Solver.
	 */
	public String getSolver(){
		return (String) lppanel.solverbox.getSelectedItem();
	}
	
	/**
	 * Method to get the correct language ResourceBundle.
	 * @return ResourceBundle for the correct language.
	 */
	public ResourceBundle getResourceBundle(){
		return languageResource;
	}
	
	/**
	 * Get the dialog.
	 * @return SettingsDialog.
	 */
	public SettingsDialog getDialog(){
		return dialog;
	}
}