//: /be/optiloading/ship/IntactStability.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.ship;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.ResourceBundle;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import be.optiloading.helpers.table.BooleanRenderer;
import be.optiloading.helpers.table.NumberRenderer;
import be.optiloading.settings.Settings;

/**
 * IntactStabilityData class which contains displays if intact stability
 * conditions are met.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class IntactStability extends JPanel {
	private JTable table;
	private JScrollPane scroll;
	private String[] columnToolTips;
	
	/**
	 * Constructor.
	 */
	public IntactStability(){
		super(new BorderLayout());
		ResourceBundle languageResource = Settings.getInstance().getResourceBundle();
		columnToolTips = new String[4];
		columnToolTips[0] = languageResource.getString("desTooltip");
		columnToolTips[1] = languageResource.getString("critTooltip");
		columnToolTips[2] = languageResource.getString("valTooltip");
		columnToolTips[3] = languageResource.getString("compliesTooltip");
		
		setBorder(new CompoundBorder(new TitledBorder(new EtchedBorder(),
				"Intact Stability"), new EmptyBorder(5, 5, 0, 5)));
		IntactStabTableModel model = new IntactStabTableModel();
		table = new JTable(model){
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column){
		        Component returnComp = super.prepareRenderer(renderer, row, column);
		        Color alternateColor = new Color(236,243,254);
		        Color whiteColor = Color.WHITE;
		        if (!returnComp.getBackground().equals(getSelectionBackground())){
		            Color bg = (row % 2 == 0 ? alternateColor : whiteColor);
		            returnComp .setBackground(bg);
		            bg = null;
		        }
		        return returnComp;
			}
		
			public void changeSelection(int rowIndex,
                    int columnIndex,
                    boolean toggle,
                    boolean extend){}
			
			 protected JTableHeader createDefaultTableHeader() {
			        return new JTableHeader(columnModel) {
			            public String getToolTipText(MouseEvent e) {
			                java.awt.Point p = e.getPoint();
			                int index = columnModel.getColumnIndexAtX(p.x);
			                int realIndex = 
			                        columnModel.getColumn(index).getModelIndex();
			                return columnToolTips[realIndex];
			            }
			        };
			    }
		};
		table.getTableHeader().setReorderingAllowed(false);
		((DefaultTableCellRenderer)table.getTableHeader().getDefaultRenderer())
		.setHorizontalAlignment(SwingConstants.CENTER);
		table.setDefaultRenderer(float.class, new NumberRenderer(3));
		table.setDefaultRenderer(boolean.class, new BooleanRenderer(true));
		
		TableColumn col = table.getColumnModel().getColumn(0);
		col.setMinWidth(350);
		col = table.getColumnModel().getColumn(1);
		col.setMinWidth(50);
		col = table.getColumnModel().getColumn(2);
		col.setMinWidth(40);
		col = table.getColumnModel().getColumn(3);
		col.setMinWidth(40);
		
		table.setRowHeight(22);
		
		scroll = new JScrollPane(table, 
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		add(BorderLayout.CENTER,scroll);
		scroll.addComponentListener(new ScrollResizeListener());
	}
	
	/**
	 * Method to update the intact stability table.
	 */
	public void update(){
		((IntactStabTableModel) table.getModel()).fireTableDataChanged();
	}
	
	//Resize listener for scrollPane
	private class ScrollResizeListener extends ComponentAdapter {
        public void componentResized(ComponentEvent e){
          if (table.getPreferredSize().width <=
          	scroll.getViewport().getExtentSize().width)
          {
            table.setAutoResizeMode(
              JTable.AUTO_RESIZE_ALL_COLUMNS);
          } else {
          	table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
          }
        }
      }
	
	//TableModel for intact stability table
	private class IntactStabTableModel extends AbstractTableModel{
		private String[] columnNames;
		private ArrayList<Criteria> rowList;
		
		public IntactStabTableModel(){
			ResourceBundle languageResource = Settings.getInstance().getResourceBundle();
			
			columnNames = new String[4];
			columnNames[0] = languageResource.getString("description");
			columnNames[1] = languageResource.getString("criterion");
			columnNames[2] = languageResource.getString("value");
			columnNames[3] = languageResource.getString("complies");

			rowList = new ArrayList<Criteria>();
			rowList.add(new Criteria(languageResource.getString("angleOfFlooding"),
					ShipParticular.ANGLE_OF_FLOODING,
					0f,Criteria.NONE));
			rowList.add(new Criteria(languageResource.getString("angleOfLoll"),
					ShipParticular.ANGLE_OF_LOLL,
					0f,Criteria.NONE));
			rowList.add(new Criteria(languageResource.getString("minGM"),
					ShipParticular.GM,
					0.15f,Criteria.MIN));
			rowList.add(new Criteria(languageResource.getString("minArea30"),
					ShipParticular.AREA_30,
					0.055f,Criteria.MIN));
			rowList.add(new Criteria(languageResource.getString("minArea40"),
					ShipParticular.AREA_40,
					0.09f,Criteria.MIN));
			rowList.add(new Criteria(languageResource.getString("minArea30to40"),
					ShipParticular.AREA_30TO40,
					0.03f,Criteria.MIN));
			rowList.add(new Criteria(languageResource.getString("minMaxGZAngle"),
					ShipParticular.MAX_GZ_ANGLE,
					30f,Criteria.MIN));
			rowList.add(new Criteria(languageResource.getString("minGZat30"),
					ShipParticular.GZ_30,
					0.2f,Criteria.MIN));
		}
		
		public Class<?> getColumnClass(int arg0) {
			Class<?> returnvalue = null;
			
			switch(arg0){
			case 0: returnvalue = String.class; break;
			case 1: returnvalue = float.class; break;
			case 2: returnvalue = float.class; break;
			case 3: returnvalue = boolean.class; break;
			}
			
			return returnvalue;
		}

		public int getColumnCount() {
			return columnNames.length;
		}

		public String getColumnName(int arg0) {
			return columnNames[arg0];
		}

		public int getRowCount() {
			return rowList.size();
		}

		public Object getValueAt(int row, int column) {
			return ((Criteria) rowList.get(row)).get(column);
		}

		public boolean isCellEditable(int arg0, int arg1) {
			return false;
		}
	}
	
	// Criteria's are stored in a separate class that is used by the 
	// intact stability TableModel
	private class Criteria{
		private String discription;
		private ShipParticular shipparticular;
		private float criteria;
		private int criteriatype;
		
		public static final int MAX = 0;
		public static final int MIN = 1;
		public static final int NONE = 2;
		
		
		public Criteria(String discription, ShipParticular shipparticular,
				float criteria, int criteriatype){
			this.discription = discription;
			this.shipparticular = shipparticular;
			this.criteria = criteria;
			this.criteriatype = criteriatype;
		}
		
		public Object get(int i){
			Object returnvalue = null;
			switch(i){
			case 0: returnvalue = discription; break;
			case 1: returnvalue = criteria; break;
			case 2: 
				returnvalue = ShipData.getInstance().getShipParticular(shipparticular); 
				break;
			case 3: 
				switch(criteriatype){
				case 0:
					if(ShipData.getInstance().getShipParticular(shipparticular)>criteria){
						returnvalue = false;
					} else {
						returnvalue = true;
					}
					break;
				case 1:
					if(ShipData.getInstance().getShipParticular(shipparticular)<criteria){
						returnvalue = false;
					} else {
						returnvalue = true;
					}
					break;
				case 2:
					returnvalue = true;
					break;
				}
				break;
			}
			return returnvalue;
		}
	}
}
