//: /be/optiloading/tank/TankList.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.tank;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.JOptionPane;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import be.optiloading.gui.MainFrame;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;

/**
 * TankList class read tank information from xml file and stowes the 
 * list of tanks as an ArrayList and has extra functions to get 
 * information on all the tanks (total weight, volume, moments...)
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings({ "serial" })
public class TankList extends ArrayList<Tank>{
	private boolean cargo;
	private float ballast;
	private float maxballast;
	
	/**
	 * Constructor of the TankList. Builds up the ArrayList<Tank>.
	 * 
	 * @param xmlFile File which contains tank information.
	 * @param cargo Boolean value which is true if this is a cargoTankList.
	 */
	public TankList(String xmlFile, boolean cargo){
		this.cargo = cargo;
		try {
			InputStream inputSource = this.getClass().
			getResource(xmlFile).openStream();
			
			SAXParserFactory factory = SAXParserFactory.newInstance();
			factory.setValidating(true);
			factory.setNamespaceAware(false);
			
			SAXParser parser = factory.newSAXParser();
			parser.parse(inputSource, new TankHandler(this));
		
		} catch (Exception e) {
			ShipData.getInstance().parseClose();
		}
	}
	
	//SAX document handler to handle the callback.
	private class TankHandler extends DefaultHandler{
		ArrayList<Tank> tankList;
		ArrayList<TankRecord> currentTankData;
		
		String tankid;
		String shortid;
		float maxvolume;
		float defaultdensity;
		float aftbound;
		float fwdbound;
		
		float ullage;
		float volume;
		float lcg;
		float vcg;
		float tcg;
		float fsm;
		
		float current;
		boolean floatvalue;
		private StringBuffer buf;
		
		public TankHandler(ArrayList<Tank> tankList){
			this.tankList = tankList;
			floatvalue = false;
		}

		public void startElement(String uri, String name, String qname,
				Attributes att) throws SAXException {
			if(qname.equals("tank")){
				currentTankData = new ArrayList<TankRecord>();
				tankid = att.getValue("id");
				shortid = att.getValue("shortid");
				maxvolume = Float.valueOf(att.getValue("maxvolume"));
				defaultdensity= Float.valueOf(att.getValue("defaultdensity"));
				aftbound= Float.valueOf(att.getValue("aftbound"));
				fwdbound = Float.valueOf(att.getValue("fwdbound"));
				if(cargo){
					ShipData.getInstance().getSegregationCompList().
					addComp(att.getValue("segregationarray"));
				}
			}
			if((qname.equals("ullage")) || (qname.equals("volume")) || 
					(qname.equals("lcg")) || (qname.equals("vcg")) || 
					(qname.equals("tcg")) || (qname.equals("fsm"))){
				floatvalue = true;
			}
			buf = new StringBuffer();
		}
		
		public void characters(char[] ch, int start, int length){
			if(buf != null){
				buf.append(ch, start, length);
			}	
		}
		
		public void endElement(String uri, String name, String qname)
		throws SAXException {
			if(floatvalue == true){
				current = Float.valueOf(buf.toString());
				floatvalue = false;
			}
			if(qname.equals("record")){
				currentTankData.add(new TankRecord(ullage,volume,lcg,vcg,tcg,fsm));
			}
			if(qname.equals("tank")){
				tankList.add(new Tank(tankid, shortid, maxvolume, defaultdensity, 
						aftbound, fwdbound, currentTankData));
			}
			if(qname.equals("ullage")){
				ullage = current;
			}
			if(qname.equals("volume")){
				volume = current;
			}
			if(qname.equals("lcg")){
				lcg = current;
			}
			if(qname.equals("vcg")){
				vcg = current;
			}
			if(qname.equals("tcg")){
				tcg = current;
			}
			if(qname.equals("fsm")){
				fsm = current;
			}
		}

	}
	
	/**
	 * Get the sum of the weights of all compartments.
	 * 
	 * @return Floating point value indication the total weight of all
	 *         compartments.
	 */
	public float getTotalWeight() {
		float sum = 0;
		for (Tank x : this) {
			sum += x.getWeight();
		}
		return sum;
	}
	
	/**
	 * Get the sum of the volumes of all compartments.
	 * 
	 * @return Floating point value indication the total volumes of all
	 *         compartments.
	 */
	public float getTotalVolumes() {
		float sum = 0;
		for (Tank x : this) {
			sum += x.getVolume();
		}
		return sum;
	}
	
	/**
	 * Get the sum of the maximum volumes of all compartments.
	 * 
	 * @return Floating point value indication the total volumes of all
	 *         compartments.
	 */
	public float getTotalMaxVolumes() {
		float sum = 0;
		for (Tank x : this) {
			sum += x.getMaxvolume();
		}
		return sum;
	}

	/**
	 * Get the total longitudinal moment created by all the tanks.
	 * 
	 * @return Floating point value containing the total longitudinal moment.
	 */
	public float getLongMoment(){
		float returnvalue = 0f;
		for (Tank x : this) {
			returnvalue += (x.getWeight() * x.getLCG());
		}
		return returnvalue;
	}
	
	/**
	 * Get the total vertical moment created by all the tanks.
	 * 
	 * @return Floating point value containing the total vertical moment.
	 */
	public float getVertMoment(){
		float returnvalue = 0f;
		for (Tank x : this) {
			returnvalue += (x.getWeight() * x.getVCG());
		}
		return returnvalue;
	}
	
	/**
	 * Get the total transversal moment created by all the tanks.
	 * 
	 * @return Floating point value containing the total transversal moment.
	 */
	public float getTransMoment(){
		float returnvalue = 0f;
		for (Tank x : this) {
			returnvalue += (x.getWeight() * x.getTCG());
		}
		return returnvalue;
	}
	
	/**
	 * Get the tot free surface moment created by all the tanks.
	 * 
	 * @return Floating point value containing the total free surface moment.
	 */
	public float getFreeSurfaceMoment(){
		float returnvalue = 0f;
		for (Tank x : this) {
			returnvalue += (x.getFSM());
		}
		return returnvalue;
	}
	
	/**
	 * Get the load at a certain dist from the APP.
	 * 
	 * @param dist Distance from the APP.
	 * @return Floating point with the load in ton/m.
	 */
	public float getLoad(float dist){
		float returnvalue = 0f;
		for(Tank x: this){
			if((x.getAftbound() <= dist) && (dist <= x.getFwdbound())){
				returnvalue += x.getWeight() / (x.getFwdbound()-x.getAftbound());
			}
		}
		return returnvalue;
	}
	
	/**
	 * Empty all tanks.
	 * @param def True if density has to be set to default, otherwise false;
	 */
	public void emptyTanks(boolean def){
		for(Tank tank: this){
			if(def){
				tank.setVolume(0f, tank.getDefaultdensity());
			} else {
				tank.setVolume(0f, Settings.getInstance().getBallastDensity());
			}
			tank.setCargoid(null);
		}
	}
	
	/**
	 * Correct trim
	 * @param destrim Desired trim.
	 */
	public void correctTrim(float destrim, float maxballast){
		this.maxballast = maxballast;
		ballast = 0f;
		float trim = ShipData.getInstance().getTrim();
		float dtrim = Math.abs(trim-destrim);
		while(round(trim,3)!=round(destrim,3)){
			//Voor LCF laden
			float loadvol = ((dtrim*10)/Settings.getInstance().getBallastDensity());
			if(trim > destrim){
				if(!loadTrim(loadvol,true)){
					break;
				}
			} else {
				if(!loadTrim(loadvol,false)){
					break;
				}
			}
			trim = ShipData.getInstance().getTrim();
			dtrim = Math.abs(trim-destrim);
		}
	}
	
	/**
	 * Correct heel.
	 * @param desheel Desired heel.
	 */
	public void correctHeel(float desheel, float maxballast){
		this.maxballast = maxballast;
		ballast = 0f;
		float heel = ShipData.getInstance().getHeel();
		float dheel = Math.abs(heel-desheel);
		while(round(heel,3)!=round(desheel,3)){
			float loadvol = ((dheel*10)/Settings.getInstance().getBallastDensity());
			if(heel > desheel){
				if(!loadHeel(loadvol,true)){
					break;
				}
			} else {
				if(!loadHeel(loadvol,false)){
					break;
				}
			}
			heel = ShipData.getInstance().getHeel();
			dheel = Math.abs(heel-desheel);
		}
	}
	
	/**
	 * Correct trim and heel.
	 * @param destrim Desired trim.
	 * @param desheel Desired heel.
	 */
	public void correctBoth(float destrim, float desheel, float maxballast){
		this.maxballast = maxballast;
		ballast = 0f;
		float heel = ShipData.getInstance().getHeel();
		float dheel = Math.abs(heel-desheel);
		float trim = ShipData.getInstance().getTrim();
		float dtrim = Math.abs(trim-destrim);
		while(round(heel,3)!=round(desheel,3)){
			float loadvol = (dheel/Settings.getInstance().getBallastDensity());
			if((heel > desheel) && (trim > destrim)){
				if(!loadBoth(loadvol,true,true)){
					break;
				}
			} else if((heel > desheel) && (trim < destrim)){
				if(!loadBoth(loadvol,true,false)){
					break;
				}
			} else if((heel < desheel) && (trim > destrim)){
				if(!loadBoth(loadvol,false,true)){
					break;
				}
			} else {
				if(!loadBoth(loadvol,false,false)){
					break;
				}
			}
			heel = ShipData.getInstance().getHeel();
			dheel = Math.abs(heel-desheel);
			trim = ShipData.getInstance().getTrim();
		}
		trim = ShipData.getInstance().getTrim();
		dtrim = Math.abs(trim-destrim);
	
		while(round(trim,3)!=round(destrim,3)){
			float loadvol = (dtrim/Settings.getInstance().getBallastDensity());
			if(trim > destrim){
				if(!loadForeAft(loadvol,true)){
					break;
				}
			} else {
				if(!loadForeAft(loadvol,false)){
					break;
				}
			}
			trim = ShipData.getInstance().getTrim();
			dtrim = Math.abs(trim-destrim);
		}
		
		trim = ShipData.getInstance().getTrim();
		dtrim = Math.abs(trim-destrim);
	
		while(round(trim,3)!=round(destrim,3)){
			float loadvol = (dtrim/Settings.getInstance().getBallastDensity());
			if(trim > destrim){
				if(!loadTrimHeel(loadvol,true, desheel)){
					break;
				}
			} else {
				if(!loadTrimHeel(loadvol,false,desheel)){
					break;
				}
			}
			trim = ShipData.getInstance().getTrim();
			dtrim = Math.abs(trim-destrim);
		}
	}
	
	// Load value to correct both
	private boolean loadBoth(float addvol, boolean neg, boolean before){
		float lcf = ShipData.getInstance().getLcf();
		boolean returnvalue = true;
		int tanknr = -1;
		float currmax = 0;
		float max = 0;
		for(int k = 0; k < size(); k++){
			if(((get(k).getVolume()+addvol) < get(k).getMaxvolume()) &&
					(get(k).getTCG()!=0f) ){
				if(neg && before){
					if((- get(k).getTCG(addvol) > 0f) 
							&& ((get(k).getLCG(addvol) - lcf)>0f)){
						max = (- get(k).getTCG(addvol)) * (get(k).getLCG(addvol) - lcf);
					} else {
						max = 0;
					}
				} else if(neg && !before) {
					if((- get(k).getTCG(addvol) > 0f) 
							&& ((lcf - get(k).getLCG(addvol))>0f)){
						max = (- get(k).getTCG(addvol)) * (lcf - get(k).getLCG(addvol));
					} else {
						max = 0;
					}
				} else if(!neg && before){
					if((get(k).getTCG(addvol) > 0f) 
							&& ((get(k).getLCG(addvol) - lcf)>0f)){
						max = (get(k).getTCG(addvol)) * (get(k).getLCG(addvol) - lcf);
					} else {
						max = 0;
					}
				} else {
					if((get(k).getTCG(addvol) > 0f) 
							&& ((lcf - get(k).getLCG(addvol))>0f)){
						max = (get(k).getTCG(addvol)) * (lcf - get(k).getLCG(addvol));
					} else {
						max = 0;
					}
				}
				if(max > currmax){
					tanknr = k;
					currmax = max;
				}
			}
		}
		if(tanknr == -1){
			returnvalue = false;
		} else {
			ballast += (addvol*Settings.getInstance().getBallastDensity());
			if(ballast > maxballast){
				returnvalue = false;
			} else {
				get(tanknr).setVolume(get(tanknr).getVolume()+addvol, 
						Settings.getInstance().getBallastDensity());
			}
		}
		return returnvalue;
	}
	
	// Load the TCG = 0 tanks
	private boolean loadForeAft(float addvol, boolean before){
		float lcf = ShipData.getInstance().getLcf();
		boolean returnvalue = true;
		int tanknr = -1;
		float currmax = 0;
		float max = 0;
		for(int k = 0; k < size(); k++){
			if(((get(k).getVolume()+addvol) < get(k).getMaxvolume()) && 
					(get(k).getTCGMax() == 0f)){
				if(before){
					max = get(k).getLCG(addvol) - lcf;
				} else {
					max = lcf - get(k).getLCG(addvol);
				}
				if(max > currmax){
					tanknr = k;
					currmax = max;
				}
			}
		}
		
		if(tanknr == -1){
			returnvalue = false;
		} else {
			ballast += (addvol*Settings.getInstance().getBallastDensity());
			if(ballast > maxballast){
				returnvalue = false;
			} else {
				get(tanknr).setVolume(get(tanknr).getVolume()+addvol, 
						Settings.getInstance().getBallastDensity());
			}
		}
		return returnvalue;
	}
	
	// Load value to correct both
	private boolean loadTrimHeel(float addvol, boolean before,float desheel){
		float lcf = ShipData.getInstance().getLcf();
		boolean returnvalue = true;
		float currmax = 0;
		float max = 0;
		int tank1 = -1;
		int tank2 = -1;
		for(int k1 = 0; k1 < size(); k1++){
			for(int k2 = 0; k2 < size(); k2 ++){
				if((k1 != k2) && ((get(k1).getVolume()+addvol) < get(k1).getMaxvolume())
					&& ((get(k2).getVolume()+addvol) < get(k2).getMaxvolume()) &&
					(((get(k1).getTCG()<0)&&(get(k2).getTCG()>0)) ||  
							((get(k1).getTCG()>0)&&(get(k2).getTCG()<0))) ){
					if(before){
						max = ((get(k1).getLCG(addvol) + get(k2).getLCG(addvol))/2) - lcf;
					} else {
						max = lcf - ((get(k1).getLCG(addvol) + get(k2).getLCG(addvol))/2);
					}
					if(max > currmax){
						tank1 = k1;
						tank2 = k2;
						currmax = max;
					}
				}
			}
		}
		
		if(tank1 == -1){
			returnvalue = false;
		} else {
			int neg = -1;
			int pos = -1;
			if(get(tank1).getTCG()<0f){
				neg = tank1;
				pos = tank2;
			} else {
				neg = tank2;
				pos = tank1;
			}
			ballast += (addvol*Settings.getInstance().getBallastDensity());
			if(ballast > maxballast){
				JOptionPane.showMessageDialog(MainFrame.getInstance(), 
						Settings.getInstance().getResourceBundle().getString("maxBallast"));
				returnvalue = false;
			} else {
				for(int k = 0; k <10; k++){
					float heel = ShipData.getInstance().getHeel();
					if(heel > desheel){
						get(neg).setVolume(get(neg).getVolume()+(addvol/10), 
								Settings.getInstance().getBallastDensity());
					} else {
						get(pos).setVolume(get(pos).getVolume()+(addvol/10), 
								Settings.getInstance().getBallastDensity());
					}
				}
			}
		}
		return returnvalue;
	}
	
	// Load value to correct heel
	private boolean loadHeel(float addvol, boolean neg){
		boolean returnvalue = true;
		int tanknr = -1;
		float currmax = 0;
		float max = 0;
		for(int k = 0; k < size(); k++){
			if((get(k).getVolume()+addvol) < get(k).getMaxvolume()){
				if(neg){
					max = - get(k).getTCG(addvol);
				} else {
					max = get(k).getTCG(addvol);
				}
				if(max > currmax){
					tanknr = k;
					currmax = max;
				}
			}
			
		}
		if(tanknr == -1){
			returnvalue = false;
		} else {
			ballast += (addvol*Settings.getInstance().getBallastDensity());
			if(ballast > maxballast){
				JOptionPane.showMessageDialog(MainFrame.getInstance(), 
						Settings.getInstance().getResourceBundle().getString("maxBallast"));
				returnvalue = false;
			} else {
				get(tanknr).setVolume(get(tanknr).getVolume()+addvol, 
						Settings.getInstance().getBallastDensity());
			}
		}
		return returnvalue;
	}
	
	// Load value to correct trim
	private boolean loadTrim(float addvol, boolean before){
		float lcf = ShipData.getInstance().getLcf();
		boolean returnvalue = true;
		int tanknr = -1;
		float currmax = 0;
		float max = 0;
		for(int k = 0; k < size(); k++){
			if((get(k).getVolume()+addvol) < get(k).getMaxvolume()){
				if(before){
					max = get(k).getLCG(addvol) - lcf;
				} else {
					max = lcf - get(k).getLCG(addvol);
				}
				if(max > currmax){
					tanknr = k;
					currmax = max;
				}
			}
			
		}
		if(tanknr == -1){
			returnvalue = false;
		} else {
			ballast += (addvol*Settings.getInstance().getBallastDensity());
			if(ballast > maxballast){
				JOptionPane.showMessageDialog(MainFrame.getInstance(), 
						Settings.getInstance().getResourceBundle().getString("maxBallast"));
				returnvalue = false;
			} else {
				get(tanknr).setVolume(get(tanknr).getVolume()+addvol, 
						Settings.getInstance().getBallastDensity());
			}
		}
		return returnvalue;
	}
	
	// Round value to indicated round number
	private float round(float value, int round){
		value *= Math.pow(10, round);
		value = Math.round(value);
		value /= Math.pow(10, round);
		return value;
	}
	
	/**
	 * Get a sorted array with all the maximum volumes.
	 * @return Floating point array.
	 */
	public float[] getMaxVolsArray(){
		float[] returnvalue = new float[size()];
		for(int i = 0; i < returnvalue.length; i++){
			returnvalue[i] = get(i).getMaxvolume();
		}
		Arrays.sort(returnvalue);
		return returnvalue;
	}
	
	/**
	 * Set the default max % for all tanks in TankList.
	 * @param factor Factor to set.
	 */
	public void setDefaultMaxFactor(float factor){
		for(Tank x: this){
			x.setMaxfactor(factor);
		}
	}
	
	/**
	 * Change the cargo id in the cargo tank list.
	 * @param old The old String id.
	 * @param newS The new String id.
	 */
	public void changeID(String old, String newS){
		for(Tank x: this){
			if((x.getCargoid() != null) &&(x.getCargoid().equals(old))){
				x.setCargoid(newS);
			}
		}
	}
}
