//: /be/optiloading/tank/TankModel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.tank;

import java.text.NumberFormat;
import java.util.ResourceBundle;

import javax.swing.JOptionPane;
import javax.swing.table.AbstractTableModel;

import be.optiloading.gui.MainFrame;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;

/**
 * TankModel is a TableModel to build tables out of the TankList objects.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class TankModel extends AbstractTableModel{
	private TankList tankList;
	private boolean editable;
	private boolean cargo;
	private String[] columnNames;
	private NumberFormat formatter;
	private ResourceBundle languageResource;
	
	/**
	 * Constructor to make up the model.
	 * 
	 * @param tankList The TankList for which the model will be build.
	 * @param editable Boolean indicating if the user may edit values.
	 * @param cargo Boolean indicating if this is a table for cargo tanks.
	 */
	public TankModel(TankList tankList, boolean editable, boolean cargo){
		languageResource = Settings.getInstance().getResourceBundle();
		String[] normaltanks = {"compartment","volume","density","percentageFull",
				"weight","ullage","maxPercentage","maxVolume","vcg","lcg","tcg","fsm"};
		String[] cargotanks = {"compartment","cargoid","volume","density","percentageFull",
				"weight","ullage","maxPercentage","maxVolume","vcg","lcg","tcg","fsm"};
		String[] uselist = null;
		if(cargo){
			uselist = cargotanks;
			columnNames = new String[cargotanks.length];
		} else {
			uselist = normaltanks;
			columnNames = new String[normaltanks.length];
		}
		for(int i=0; i < uselist.length;i++){
			columnNames[i] = languageResource.getString(uselist[i]);
		}
		this.cargo = cargo;
		this.tankList = tankList;
		this.editable = editable;
		formatter = NumberFormat.getInstance();
        formatter.setMaximumFractionDigits(3);
        formatter.setMinimumFractionDigits(2);
	}
	
	/**
	 * Get the column name.
	 * 
	 * @param column Specification of which column.
	 */
	public String getColumnName(int column){
		return columnNames[column];
	}

	/**
	 * Get the number of columns.
	 * 
	 * @return Integer indicating the number of columns.
	 */
	public int getColumnCount() {
		return columnNames.length;
	}

	/**
	 * Get the number of rows.
	 * 
	 * @return Integer indicating the number of rows.
	 */
	public int getRowCount() {
		return tankList.size();
	}
	
	/**
	 * Get the class of of a certain columns.
	 * 
	 * @param column Column for which you want the class.
	 * @return Class of the certain column.
	 */
	public Class<?> getColumnClass(int column){
		int i = 0;
		if(cargo){
			i = 1;
		}
		if(column > i){
			return Float.class;
		} else {
			return String.class;
		}
	}

	/**
	 * Get the value of a specific cell.
	 * 
	 * @param row Row of the cell.
	 * @param column Column of the cell.
	 * @return Object value of the cell.
	 */
	public Object getValueAt(int row, int column) {
		Tank current = tankList.get(row);
		Object returnvalue = null;
		switch(column){
		case 0: 
			returnvalue = current.getTankid();
			break;
		case 1: 
			if(cargo){
				returnvalue = current.getCargoid();
			} else{
				returnvalue = current.getVolume();
			}
			break;
		case 2:
			if(cargo){
				returnvalue = current.getVolume();
			} else{
				returnvalue = current.getDensity();
			}
			break;
		case 3:
			if(cargo){
				returnvalue = current.getDensity();
			} else{
				returnvalue = (current.getVolume()/current.getMax());
			}
			break;
		case 4:
			if(cargo){
				returnvalue = (current.getVolume()/current.getMax());
			} else{
				returnvalue = current.getWeight();
			}
			break;
		case 5:
			if(cargo){
				returnvalue = current.getWeight();
			} else{
				returnvalue = current.getUllage();
			}
			break;
		case 6:
			if(cargo){
				returnvalue = current.getUllage();
			} else{
				returnvalue = current.getMaxfactor();
			}
			break;
		case 7:
			if(cargo){
				returnvalue = current.getMaxfactor();
			} else{
				returnvalue = current.getMaxvolume();
			}
			break;
		case 8:
			if(cargo){
				returnvalue = current.getMaxvolume();
			} else{
				returnvalue = current.getVCG();
			}
			break;
		case 9:
			if(cargo){
				returnvalue = current.getVCG();
			} else{
				returnvalue = current.getLCG();
			}
			break;
		case 10:
			if(cargo){
				returnvalue = current.getLCG();
			} else{
				returnvalue = current.getTCG();
			}
			break;
		case 11:
			if(cargo){
				returnvalue = current.getTCG();
			} else{
				returnvalue = current.getFSM();
			}
			break;
		case 12:
			returnvalue = current.getFSM();
		}
		return returnvalue;
	}
	
	/**
	 * Set the value of a specific cell.
	 * 
	 * @param row Row of the cell.
	 * @param column Column of the cell.
	 */
	public void setValueAt(Object objectvalue, int row, int column){
		
		
		Tank current = tankList.get(row);
		Number number = null;
		float value = 0f;
		if(cargo){
			column--;
			if(column == 0){
				current.setCargoid((String) objectvalue);
			} else {
				number = (Number) objectvalue;
				value = number.floatValue();
			}
		} else {
			number = (Number) objectvalue;
			value = number.floatValue();
		}
		
		if(value < 0f){
			JOptionPane.showMessageDialog(MainFrame.getInstance(),
					languageResource.getString("messageNegative"));
			return;
		}
		
		ShipData data = ShipData.getInstance();
		
		float maxvolume = current.getMaxvolume();
		float maxullage = current.getMaxUllage();
		float maxweight = (data.getDeadweightleft() + current.getWeight());

		switch(column){
		case 1:
			if(value > maxvolume){
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(
						languageResource.getString("messageMaxVolume2"), 
						formatter.format(maxvolume)));
						
						
			} else if ((value*current.getDensity()) > maxweight) {
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(
						languageResource.getString("messageMaxWeight"), 
						formatter.format(maxweight),
						formatter.format(data.getMaxdeadweight())));
			} else {
				current.setVolume(value);
			}
			break;
		case 2:
			if((value*current.getVolume())> maxweight){
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(
						languageResource.getString("messageMaxWeight"), 
						formatter.format(maxweight),
						formatter.format(data.getMaxdeadweight())));
			} else {
				current.setDensity(value);
			}
			break;
		case 3:
			if((value*current.getMax()*current.getDensity()) > maxweight){
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(
						languageResource.getString("messageMaxWeight"), 
						formatter.format(maxweight),
						formatter.format(data.getMaxdeadweight())));
			} else if ((value*current.getMax())>maxvolume){
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(
						languageResource.getString("messageMaxVolume2"), 
						formatter.format(maxvolume)));
			} else {
				current.setVolume(value*current.getMax());
			}
			break;
		case 4:
			if(value > maxweight){
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(
						languageResource.getString("messageMaxWeight"), 
						formatter.format(maxweight),
						formatter.format(data.getMaxdeadweight())));
			} else if((value/current.getDensity())>maxvolume) {
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(
						languageResource.getString("messageMaxVolume2"), 
						formatter.format(maxvolume)));
			} else {
				current.setWeight(value);
			}
			break;
		case 5:
			if(value > maxullage){
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(
						languageResource.getString("messageMaxUlllage"), 
						formatter.format(maxullage)));
			} else {
				float oldullage = current.getUllage();
				current.setUllage(value);
				
				if(current.getWeight()> maxweight){
					JOptionPane.showMessageDialog(MainFrame.getInstance(),
							String.format(
							languageResource.getString("messageMaxWeight"), 
							formatter.format(maxweight),
							formatter.format(data.getMaxdeadweight())));
					current.setUllage(oldullage);
				}
			}
			break;
		case 6:
			if(value <= 1f){
				float newvol = ShipData.getInstance().getCargoTankList()
					.getTotalMaxVolumes() - current.getMaxvolume() 
					+ (value * current.getMax());
				if((cargo) && (newvol < ShipData.getInstance().getCargoVolume())){
					JOptionPane.showMessageDialog(MainFrame.getInstance(),
							languageResource.getString("messageAdjustCargo"));
				} else {
					if(value < (current.getVolume()/current.getMax())){
						current.setVolume(value*current.getMax());
					}
					current.setMaxfactor(value);
				}
			}
			break;
		}
		fireTableDataChanged();

		MainFrame.getInstance().updateInterface();
		data = null;
		current = null;
	}
	
	/**
	 * Indicated if a cell is editable or not. Depends on the editable value
	 * given in the constructor.
	 * 
	 * @param row Row of the cell.
	 * @param column Column of the cell.
	 */
	public boolean isCellEditable(int row, int column){
		int editc = 6;
		if(cargo){
			editc = 7;
		}
		if(((editable == true) && (column > 0) && (column < editc)) ||
				(column == editc)){
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Is this a cargo table?
	 * @return True if it is, otherwise false.
	 */
	public boolean isCargo() {
		return cargo;
	}

	/**
	 * Make tank table editable.
	 * @param editable True if editable, otherwise false.
	 */
	public void setEditable(boolean editable){
		this.editable = editable;
	}
	
}
